#!/usr/dist/pkgs/perl,v5.003/5bin.sun4/perl
######################################################################
# Sample script using the SNMP_SIMPLE module.
#
# The purpose of this script is to demonstrate the features of the
# SNMP_Simple perl module.  Explanations and sample code for most
# of the functions are given.
#
# This script assumes that it is in the same directory as all the
# modules, and that two parameters is passed in - the hostname to 
# query and the community string to do the query with.
#
######################################################################

require 5;
BEGIN {
    ($workdir) = ($0 =~ /(.*)\/.*/);
}
use lib ("$workdir/../lib");
use BER;
use SNMP_Session;
use SNMP_Simple;
use SNMP_MIB;

# Uncomment the below to turn on debug messages
#$SNMP_Simple::debug=1;

# Comment out the below to turn on warnings from the SNMP_Simple module
$SNMP_Simple::suppress_warnings=0;

# Comment out the below to turn on warnings from the SNMP_Session module
$SNMP_Session::suppress_warnings=0;

# The below initalizes the OID mappings for the SNMP_Simple library.
#SNMP_MIB::loadoids("/home/hippie/projects/em/mibs/oids", "RFC1213");
#$SNMP_MIB::debug = 10;
SNMP_MIB::loadmib("$workdir/../mibs", "RFC1213-MIB.my", "ianaiftype.mib", "IF-MIB.my");
#SNMP_MIB::dump_names();
#SNMP_MIB::dump_oids();
#SNMP_MIB::dump_enums();

my($hostname, $community) = @ARGV;

# Create the session (open the socket)
my($session) = SNMPv2c_Simple->open($hostname, $community);
die unless defined($session);

# Sample of snmpget
# 
# snmpget is the most simple of the SNMP get functions.  It simply
# takes a list of variables and returns a list of values, one value
# for each variable passed in.  Values are returned in the same order
# as the variables passed in.  If an error was encountered for a
# variable, an 'undef' value is returned in its place.

print "\nsnmpget:\n";
my($sysDescr, $foo, $sysContact, $sysLocation, $sysName, $sysUpTime,
   $sysObjectID, $interfaceNumber)
    = $session->snmpget('sysDescr', 'foo', 'sysContact', 'sysLocation',
			'sysName', 'sysUpTime', 'sysObjectID',
			'ifNumber');

my($sysObjectName) = SNMP_MIB::oid2name($sysObjectID);
print <<EOF;
Name\t\t$sysName
Description\t$sysDescr
Contact\t\t$sysContact
Location\t$sysLocation
Uptime\t\t$sysUpTime
ObjectID\t$sysObjectID: $sysObjectName
Interfaces\t$interfaceNumber
EOF

# Sample of snmpgeta
#
# snmpgeta does much the same thing as snmpget.  It takes a list of
# variables and returns a associative array of values, where the 
# variable name maps to the value returned.  No entry is made in the
# associative array for invalid variable names or for error returns.

print "\nsnmpgeta:\n";
my %outvars = $session->snmpgeta('sysDescr', 'sysContact',
				 'sysLocation', 'sysName',
				 'sysUpTime', 'sysObjectID',
				 'ifNumber');
my $i;
foreach $i (sort byOID keys %outvars) {
    print "$i\t$outvars{$i}\n";
}

#sleep 5;

# Sample of snmpgettable
#
# snmpgettable takes a variable name and returns a list of all the
# values for that variable.  The intention is that the variable name
# is the name of a column in a table, and the result is a list of
# all the values of that variable for each row in the table.  No
# sanity checking is done, values are put on the list in FIFO
# order.  No guarantee is made that the list index will equal the row
# index.

print "\nsnmpgettable:\n";
my @outvars = $session->snmpgettable('ifDescr');
my $i;
foreach $i (@outvars) {
    print "$i\n";
}

#sleep 5;


#Sample of snmpgettablea
#
# snnmpgettablea takes a variable name and returns an associative
# array with all the values for that variable.  The key in the
# associative array is an OID, the value is the value retrived for
# that OID.

print "\nsnmpgettablea:\n";
my %outvars = $session->snmpgettablea('ifDescr');
my $i;
foreach $i (sort byOID keys %outvars) {
    print "$i -> $outvars{$i}\n";
}

#sleep 5;

#Sample of snmpgettablean
#
# snnmpgettablea takes a variable name and returns an associative
# array with all the values for that variable.  The key in the
# associative array is an OID, the value is the value retrived for
# that OID.

print "\nsnmpgettablean:\n";
my %outvars = $session->snmpgettablean('ifDescr');
my $i;
foreach $i (sort { $a <=> $b } keys %outvars) {
    print "$i -> $outvars{$i}\n";
}

#sleep 5;

#sample of snmpgettabkek
#
# snmpgettablek takes the following parameters:
#     Name of a table element (as in 'ifEntry');
#     List of table columns (as in 'ifIndex', 'ifDescr', ...)
# And returns a list where each element in the list is a hash mapping
# variable names to values.  The table is assumed to have a simple
# (one element) index, and each element is put into the correct list
# element based on that index.
#
# Sample - Call snmpgettablek like this:
#
# my @outvars = $session->snmpgettablek('ifEntry', 'ifDescr', 'ifType');
#
# and what will be returned is an array where element 0 is undefined,
# elements 1 through $#outvars are hashes with three keys ('index',
# 'ifDescr' and 'ifType'

print "\nsnmpgettablek:\n";
my @outvars = $session->snmpgettablek('ifDescr', 'ifType', 'ifSpeed', 'ifInOctets', 'ifOutOctets', 'ifHCInOctets', 'ifHCOutOctets');
my $i;
my %IANAifType = SNMP_MIB::getEnum('IANAifType');
foreach $i (1 .. $#outvars) {
    my $ifType = $outvars[$i]->{'ifType'};
    $ifType = $IANAifType{$ifType} if defined($IANAifType{$ifType});
    print "$i -> $outvars[$i]->{'ifDescr'}\t$ifType\t$outvars[$i]->{'ifSpeed'}\t$outvars[$i]->{'ifInOctets'}\t$outvars[$i]->{'ifHCInOctets'}\t$outvars[$i]->{'ifOutOctets'}\t$outvars[$i]->{'ifHCOutOctets'}\n";
}

#sleep 5;

#sample of snmpgettabkeka
#
# snmpgettableka takes the following parameters:
#     Name of a table element (as in 'ifEntry');
#     List of table columns (as in 'ifIndex', 'ifDescr', ...)
# And returns an associtave array where each element is a hash mapping
# variable names to values, and the key is the index in the table for
# those elements. 
#
# Sample - Call snmpgettableka like this:
#
# my @outvars = $session->snmpgettableka('ifEntry', 'ifDescr', 'ifType');
#
# and what will be returned is an associative array with a key for
# each valid row in the interfaces table, and each element is a hash
# with three keys ('index', 'ifDescr' and 'ifType')

print "\nsnmpgettableka:\n";
my %outvars = $session->snmpgettableka(1, 'ifDescr', 'ifAdminStatus', 'ifOperStatus');
my $i;
my %ifAdminStatus = SNMP_MIB::getEnum('ifAdminStatus');
my %ifOperStatus = SNMP_MIB::getEnum('ifOperStatus');
foreach $i (sort byOID keys %outvars) {
    my $adminStatus = $outvars{$i}->{'ifAdminStatus'};
    $adminStatus = $ifAdminStatus{$adminStatus} if defined($ifAdminStatus{$adminStatus});
    my $operStatus = $outvars{$i}->{'ifOperStatus'};
    $operStatus = $ifOperStatus{$operStatus} if defined($ifOperStatus{$operStatus});
    print "$i -> $outvars{$i}->{'ifDescr'}\t$adminStatus\t$operStatus\n";
}
$session->close();

exit 0;
