#!/usr/local/bin/perl
#
#    nmis.pl - NMIS Perl Program - Network Mangement Information System
#    Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au/nmis
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#*****************************************************************************
# Check if there are any arguements
&checkArgs;

# Auto configure to the <nmis-base>/lib and <nmis-base>/files/nmis.conf
use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "/usr/local/rrdtool/lib/perl"; 
# 
#****** Shouldn't be anything else to customise below here *******************
# best to customise in the nmis.conf file.
#

require 5;

use Time::HiRes;
my $startTime = Time::HiRes::time();

use strict;
use csv;
use BER;
use SNMP_Session;
use SNMP_MIB;
use SNMP_Simple;
use SNMPv2c_Simple;
use RRDs;
use NMIS;
use func;
use ip;
use ping;
use notify;

# Global Interpreted Variables ie stuff we figured out. 
my $sysObjectName;
my $ifIndex;
my $ifSpeed;

# Variable used to control writing the system file to ensure that it doesn't get blanked.
my $writeSystem = "false";

# Time Variables
my $time;
my $datestamp;
my $datestamp_2;
my $endTime;
# Global SNMP Variables once we get em keep em global
my %snmpTable;

# Variables for command line munging
my %nvp = getArguements(@ARGV);

# See if customised config file required.
my $conf;
if ( $nvp{file} ne "" ) { $conf = $nvp{file}; }
else { $conf = "nmis.conf"; }
my $configfile = "$FindBin::Bin/../conf/$conf";
if ( -r $configfile ) { loadConfiguration($configfile); }
else { die "Can't access configuration file $configfile.\n"; }

#Load the configuration
loadConfiguration($configfile);

# all arguments are now stored in nvp (name value pairs)
my $type=$nvp{type};
my $graph=$nvp{graph};
my $collect=$nvp{collect};
my $node_search=$nvp{node};
my $rmefile=$nvp{rmefile};

# Set debugging level.
my $debug = setDebug($nvp{debug});
$NMIS::debug = $debug;

# Create a hash for storing runReachability stats
my %reach;

# Create a hash for storing interface stats
my %ifStats;

# An array of Interface Status for AdminStatus and OperStatus
my @interfaceStatus = ("null","up","down","testing","unknown","dormant","notPresent");

# Set the default collect option
if ($collect eq "") { $collect = "true"; }

# Set the default graph option
if ($graph eq "") { $graph = "false"; }

# Other Global Variables
my @interfaceTable;
my $ping;
my $timeout = 10;
my $snmpresult;
my $run;
my $statsDir;
my $onenode;
my $index;
my $nodecount;

# The node to turn on debugging for no matter what
my $debug_watch = "";
my $debug_global = $debug;

# How long to run for in seconds thus 30 * 60 is 1800 seconds
my $start_time = time;
my $run_time = 60 * 60;

if ($debug) { print returnTime ." Starting $0 $NMIS::VERSION: type=$type file=$configfile node=$node_search run_time=$run_time\n"; }

# Load the configuration and the node file!!!!!!!
loadConfiguration($configfile);
loadNodeDetails;

# qr creates FAST REGEX for patterns used all the time.
my $qr_int_stats = qr/$NMIS::config{int_stats}/i;
my $qr_hc_model = qr/$NMIS::config{hc_model}/i;
my $qr_ignore_up_down_ifDescr = qr/$NMIS::config{ignore_up_down_ifDescr}/i;
my $qr_ignore_up_down_ifType = qr/$NMIS::config{ignore_up_down_ifType}/i;
my $qr_no_collect_ifDescr_gen = qr/$NMIS::config{no_collect_ifDescr_gen}/i;
my $qr_no_collect_ifDescr_atm = qr/$NMIS::config{no_collect_ifDescr_atm}/i;
my $qr_no_collect_ifDescr_voice = qr/$NMIS::config{no_collect_ifDescr_voice}/i;
my $qr_no_collect_ifType_gen = qr/$NMIS::config{no_collect_ifType_gen}/i;
my $qr_link_ifType = qr/$NMIS::config{link_ifType}/i;

##############################################################
# SNMP Setup, debug options, load base mibs etc.
##############################################################
# set debug on or not depending on global debug 
if ($debug > 1) {
	#You can turn off SNMP debug but still leave debug 
	#messages on just comment the following line for no 
	#SNMP debug at all
	#$SNMP_Simple::debug=1;
	
	# If you want SNMP warnings comment these lines
	#$SNMP_Simple::suppress_warnings=1;
	#$SNMP_Session::suppress_warnings=1;
}
else {
	$SNMP_Simple::suppress_warnings=1;
	$SNMP_Session::suppress_warnings=1;
	$SNMP_MIB::suppress_warnings=1;
}

# initalizes the OID mappings for the SNMP_Simple library.
if ($debug) { print returnTime." Loading All the Mibs\n"; }

if ($debug > 1) { print "\tLoading $NMIS::config{mib_root}/$NMIS::config{full_mib}\n"; }
if ( -r "$NMIS::config{mib_root}/$NMIS::config{full_mib}" ) { 
	SNMP_MIB::loadoids_file("$NMIS::config{mib_root}", "$NMIS::config{full_mib}"); 
} else { warn returnTime." nmis.pl, mib file $NMIS::config{mib_root}/$NMIS::config{full_mib} not found.\n"; }	

# If this is an update, check the config and report any errors
if ( $type eq "update" ) { &checkConfig; }

# If this is an escalation run, do it
if ( $type eq "escalate" ) { &runEscalate; }
# If this is an config run, do it
elsif ( $type eq "config" ) { &checkConfig("fix"); }
# If this is an config run, do it
elsif ( $type eq "links" ) { &runLinks; }
# If this is an apache run, do it
elsif ( $type eq "apache" ) { &printApache; }
# If this is a threshold run, do it
elsif ( $type eq "threshold" ) {
	#######################################################################
	# Do the threshold thing
	#######################################################################
	if ( $node_search eq "" ) {
		foreach $onenode ( keys (%NMIS::nodeTable) ) {
			# KS 16 Mar 02, implementing David Gay's requirement for deactiving
			# a node, ie keep a node in nodes.csv but no collection done.
			if ( $NMIS::nodeTable{$onenode}{active} ne "false" ) {
				&runThreshold($onenode);
			}
		}
	}
	else {
		&runThreshold($node_search);
	}
}      
# Read a Cisco Works file and produce an NMIS Nodes file
elsif ( $type eq "rme" ) {
	if ( $rmefile eq "" ) {
		errorQuit ("$0 the type=rme option requires a file arguement for source rme CSV file\ni.e. $0 type=rme rmefile=/data/file/rme.csv");
	}
	else {
		loadRMENodes($rmefile);
		writeNodesFile("$NMIS::config{Nodes_Table}.new");
	}
}
# Must be a collect run or something on one node
elsif ( $node_search ne "" ) {
	my $nodename = lc($node_search);
	if ( $node_search eq $NMIS::nodeTable{$node_search}{node} ) {
		&runNodeStats($node_search);
	}
	else {
		print "Invalid node $node_search: No node of that name!\n";
	}
}
# Must be a collect run or something so do all the nodes
else {
	##############################################################
	# Processing Loop for all nodes
	##############################################################	
	$index = 0;
	foreach $onenode ( keys (%NMIS::nodeTable) ) {
		# on each loop verify if runstats has been running too
		# long, each pass should always be less then 30 minutes
		if ( time - $start_time > $run_time and $type ne "update" ) {
			$start_time = returnDateStamp($start_time);
			&errorQuit ("$0 has been running for more then $run_time seconds exiting....\nStart Time $start_time. Processed $index nodes. Current node=$onenode.");
		}
		
		# This will allow debugging to be turned on for a  
		# specific node where there is a problem
		if ( $onenode eq "$debug_watch" ) { $debug = "true"; }
		else { $debug = $debug_global; }

		# KS 16 Mar 02, implementing David Gay's requirement for deactiving
		# a node, ie keep a node in nodes.csv but no collection done.
		if ( $NMIS::nodeTable{$onenode}{active} ne "false" ) {
			undef %reach;
			&runNodeStats($onenode);
			++$index;
		}
	} # foreach $onenode
}

if ($debug) { 
	$endTime = sprintf( "%.2f", Time::HiRes::tv_interval([$startTime]));
	print "\n".returnTime ." Processed $nodecount nodes current runtime $endTime seconds.\n";
};

# Couple of post processing things.
if ( $type eq "collect" ) { 
	&runMetrics; 
	&runEscalate; 
}
elsif ( $type eq "update" ) { &createInterfaceInfo; }

if ($debug) { 
	$endTime = sprintf( "%.2f", Time::HiRes::tv_interval([$startTime]));
	print "\n".returnTime ." End of $0 Ran for $endTime seconds.  BYE!\n\n";
};

exit 0;

sub runNodeStats {
	my $runnode = shift;

	my $pingresult;
	my $timestring;
	my $nodeCollect;
	my $start_run;
	my $end_run;

	++$nodecount;

	if ($debug) { 
		$start_run = Time::HiRes::time();
		print "\n";
		print returnTime." Starting runNodeStats for $runnode\n";
	};

	undef %reach;
		
	## KS 1/4/2001 - Load the node file if it exists.
	if ( -r "$NMIS::config{'<nmis_var>'}/$runnode.dat" ) {
		loadSystemFile($runnode);
	}
	if ($debug) { 
		print returnTime." node=$runnode role=$NMIS::systemTable{roleType} type=$NMIS::systemTable{nodeType}\n";
		print "\tvendor=$NMIS::systemTable{nodeVendor} model=$NMIS::systemTable{nodeModel} interfaces=$NMIS::systemTable{ifNumber}\n";
	}

	#######################################################################
	# Lets do a ping to see if the device is reachable or not
	#######################################################################
	### AS 8 June 2002 - Removing legacy interface and health types
	if ( $type eq "collect" or $type eq "update" or $type eq "threshold" ) {
		$pingresult = &runPing($runnode);
	}

	#######################################################################
	# Are we supposed to collect on this node set a temp boolean for this
	#######################################################################
	if ( $collect eq "true" and $NMIS::nodeTable{$runnode}{collect} eq "true" and $pingresult == 100 ) { 
		$nodeCollect = "true"; 
	}
	else { $nodeCollect = "false"; }
	 
	#######################################################################
	# Load the system info which is kept on file if it exists otherwise create it.
	#######################################################################
	if ( -r "$NMIS::config{'<nmis_var>'}/$runnode.dat" 
		and $NMIS::nodeTable{$runnode}{collect} eq "true" 
		and $pingresult == 100 
		and $type ne "update"
	) {
		loadSystemFile($runnode);
		&updateUptime($runnode, $pingresult);
	}
	elsif ( not -r "$NMIS::config{'<nmis_var>'}/$runnode.dat" ) {
		if ($debug) { print returnTime." No system file exists creating one.\n"; }
		&createSystemFile($runnode, $pingresult);
	}
	
	if ($debug>2) {
		if ( $NMIS::systemTable{sysDescr} ne "" ) {
			print "$NMIS::systemTable{sysDescr}\n";
			print "$NMIS::systemTable{sysUpTime}, $NMIS::systemTable{sysObjectID}, $NMIS::systemTable{sysServices}\n";
		}
	}

	if ($debug>8) { &runEventDebug; }
	
	#######################################################################
	# Now depending on the type of job we are doing.
	# Just update the interface file for the device.
	#######################################################################
	if ( $runnode eq "" or $type eq "" 
		or $type !~ /update|collect/
	) {
		&errorQuit("ERROR: no node or bad type, node=$runnode type=$type collect=$collect nodeCollect=$nodeCollect");
	}

	#######################################################################
	# If there is a problem talking to node with SNMP or ICMP
	#######################################################################
	elsif ( $snmpresult != 100 and $pingresult == 100 ) {
		if ($debug) { print returnTime." $runnode SNMP is not responding skipping to next node\n"; }
		&runReachability($runnode, $pingresult);
	}
	elsif ( $pingresult != 100 ) {
		if ($debug) { print returnTime." $runnode is down not collecting skipping to next node\n"; }
		&runReachability($runnode, $pingresult);
	}

	#######################################################################
	# No collect on Node only do Reachabillity
	#######################################################################
	elsif ( $NMIS::systemTable{supported} eq "false" 
		or $NMIS::nodeTable{$runnode}{collect} eq "false" 
	) {
		&runReachability($runnode, $pingresult);
	} 
	
	#######################################################################
	# update all the interface stats
	#######################################################################
	elsif ( $type eq "update" ) {
		&runUpdate($runnode, $pingresult);
	}

	#######################################################################
	# type = collect so do health interface and reachability
	#######################################################################
	### AS 8 June 2002 - Removing legacy interface and health types
	elsif ( $type eq "collect" 
		and $NMIS::nodeTable{$runnode}{collect} eq "true" 
		#and $NMIS::systemTable{supported} eq "true" 
		and $pingresult == 100 
		and $snmpresult == 100
	) {  
		if ($debug) { print returnTime." $runnode running collect for interface and health\n"; }
		if ( $nodeCollect eq "true" ) {
			&runHealth($runnode);
		}
		&runInterface($runnode, $pingresult);
		&runReachability($runnode, $pingresult);
	} # end type eq collect
	else {
		print "Unknown ERROR. node=$runnode type=$type nodeCollect=$nodeCollect collect=$NMIS::nodeTable{$runnode}{collect} ping=$pingresult snmp=$snmpresult supported=$NMIS::systemTable{supported}\n";
		exit -1;
	}
	
	if ($debug) { 
		$end_run = sprintf( "%.2f", Time::HiRes::tv_interval([$start_run]));
		print returnTime." Ending poll of $runnode; Run Time = $end_run seconds\n";
	}
} # end runNodeStats

# Handle some simple debuging
sub errorQuit {
	my $string = shift;
	print returnDateStamp." $string\n";
	exit -1;
}

sub runInterface {
	my $node = shift;
	my $session;
	my $snmpError;
	my $message;
	my $intf;
	my %interfaceTable;
	my $createdone = "false";
	my $int_type;
	my $create1;
	my $create2;
	
	my $interfacefile = "$NMIS::config{'<nmis_var>'}/$node-interface.dat";
	
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	if ( not defined($session) ) { 
		warn returnTime." runInterface, Session is not Defined Goodly to $node.\n"; 
		goto END_runInterface;
	}
	
	# See if an interface definition file exists
	if ( -r $interfacefile ) {
		#Interface file exists load the definitions
	
		if ($debug) { print returnTime." Loading the Interface File with $NMIS::systemTable{ifNumber} entries\n"; }
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","ifIndex","\t");
	
		# This bit makes sure that the device wasn't down the last time it refreshed its interface files
		if ( not defined %interfaceTable ) {
			$message = "$node, interface file empty regenerating: $interfacefile";
			logMessage("runInterface, $message");
		    if ($debug) { print returnTime." $message\n"; }
			&createInterfaceFile($node); 
			%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
		}
	}
	else {
		# Interface file does not exist or we can't read it better create it better not be owned by someone else
		&createInterfaceFile($node); 
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
	}
	
	# Start a loop which go through the interface table

	## KS 9/4/01 Various changes to handle interfaces which get shutdown
	## during the day.
	
	#for ( $index = 1; $index <= $#interfaceTable; ++$index ) {
	# it would be nice to do some parallell interface processing here ie snmp 2 or 3 interfaces each hit.
	foreach $intf (keys %interfaceTable ) {
		# only collect on interfaces that are defined, with collection turned on globally
		# and for that interface and that are Admin UP
		if (    $collect eq "true" and 
			$interfaceTable{$intf}{ifAdminStatus} eq "up" and 
			$interfaceTable{$intf}{collect} eq "true"
			#and ( $NMIS::config{bad_interfaces} !~ /$interfaceTable{$intf}{Description}/ )
		) {
			# Check if the RRD Database exists already if not create it
			# set the database name to be the stats directory with the router name 
			# and the interface name which should have been passed in by runstats
			$create1 = &createRRDDB(type => "interface", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}, ifDescr => $interfaceTable{$intf}{ifDescr}, ifType => $interfaceTable{$intf}{ifType}, ifIndex => $interfaceTable{$intf}{ifIndex}, ifSpeed => $interfaceTable{$intf}{ifSpeed});
			if ( $interfaceTable{$intf}{ifType} =~ /$qr_int_stats/i ) {
				$create2 = &createRRDDB(type => "packets", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}, ifDescr => $interfaceTable{$intf}{ifDescr}, ifType => $interfaceTable{$intf}{ifType}, ifIndex => $interfaceTable{$intf}{ifIndex}, ifSpeed => $interfaceTable{$intf}{ifSpeed});
			} else {
				$create2 = 1;
			}
			if ( $create1 and $create2 ) {
				# Handle the HC counters for devices which support them.
				my $ifInOctets = 'ifInOctets';
				my $ifOutOctets = 'ifOutOctets';
				my $ifInUcastPkts = 'ifInUcastPkts';
				my $ifOutUcastPkts = 'ifOutUcastPkts';
				if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" 
					and $NMIS::systemTable{nodeModel} =~ /$qr_hc_model/i
				) {
					$ifInOctets = 'ifHCInOctets';
					$ifOutOctets = 'ifHCOutOctets';
					$ifInUcastPkts = 'ifHCInUcastPkts';
					$ifOutUcastPkts = 'ifHCOutUcastPkts';
				}
				if ( $interfaceTable{$intf}{ifType} =~ /$qr_int_stats/i ) {
					$int_type = "full";
					(	$ifStats{ifDescr},
						$ifStats{ifOperStatus},
						$ifStats{ifAdminStatus},
						$ifStats{ifInOctets},
						$ifStats{ifInUcastPkts},
						$ifStats{ifInNUcastPkts},
						$ifStats{ifInDiscards},
						$ifStats{ifInErrors},
						$ifStats{ifOutOctets},
						$ifStats{ifOutUcastPkts},
						$ifStats{ifOutNUcastPkts},
						$ifStats{ifOutDiscards},
						$ifStats{ifOutErrors}
					) = $session->snmpget(  
						'ifDescr'.".$interfaceTable{$intf}{ifIndex}",
						'ifOperStatus'.".$interfaceTable{$intf}{ifIndex}",
						'ifAdminStatus'.".$interfaceTable{$intf}{ifIndex}",
						$ifInOctets.".$interfaceTable{$intf}{ifIndex}",
						$ifInUcastPkts.".$interfaceTable{$intf}{ifIndex}",
						'ifInNUcastPkts'.".$interfaceTable{$intf}{ifIndex}",
						'ifInDiscards'.".$interfaceTable{$intf}{ifIndex}",
						'ifInErrors'.".$interfaceTable{$intf}{ifIndex}",
						$ifOutOctets.".$interfaceTable{$intf}{ifIndex}",
						$ifOutUcastPkts.".$interfaceTable{$intf}{ifIndex}",
						'ifOutNUcastPkts'.".$interfaceTable{$intf}{ifIndex}",
						'ifOutDiscards'.".$interfaceTable{$intf}{ifIndex}",
						'ifOutErrors'.".$interfaceTable{$intf}{ifIndex}"
					);
				} else {
					$int_type = "less";
					(	$ifStats{ifDescr},
						$ifStats{ifOperStatus},
						$ifStats{ifAdminStatus},
						$ifStats{ifInOctets},
						$ifStats{ifOutOctets}
					) = $session->snmpget(  
						'ifDescr'.".$interfaceTable{$intf}{ifIndex}",
						'ifOperStatus'.".$interfaceTable{$intf}{ifIndex}",
						'ifAdminStatus'.".$interfaceTable{$intf}{ifIndex}",
						$ifInOctets.".$interfaceTable{$intf}{ifIndex}",
						$ifOutOctets.".$interfaceTable{$intf}{ifIndex}"
					);				
				}
				if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
					$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
					$SNMP_Simple::errmsg = "";
					logMessage("runInterface, $message");
					if ($debug) { print returnTime." $message\n"; }
					goto END_runInterface;
				}
				$ifStats{ifDescr} = rmBadChars($ifStats{ifDescr});
		
				if  ( $ifStats{ifInOctets} eq "" and $ifStats{ifOutOctets} eq "" ) {
					$message = "$node, interface $interfaceTable{$intf}{ifDescr} ifIn and Out Octets are blank; maybe be an interface which does not collect stats or bad SNMP configuration or access control on the router";
					logMessage("runInterface, $message");
					if ($debug) { print returnTime." $message\n"; }
					$snmpError = "true";
				}
				else {
					$snmpError = "false";
				}
		
				if ($debug) { print "    Int=$interfaceTable{$intf}{ifDescr} type=$int_type ifIndex=$interfaceTable{$intf}{ifIndex} OperStatus=$ifStats{ifOperStatus} in=$ifStats{ifInOctets} out=$ifStats{ifOutOctets}\n"; }
				
				## KS 17 Mar 02 - Request James Norris, Optionally include interface description in messages.
				my $int_desc ;
				if ( defined $interfaceTable{$intf}{Description}
					and $interfaceTable{$intf}{Description} ne "null"
					and $NMIS::config{send_description} eq "true"
				) {
					$int_desc = " description=$interfaceTable{$intf}{Description}";
				}
				
				if ( $snmpError eq "false" ) {
					# Now Compare Real Oper Status to Admin Status if different Log
					# Interface is down but it is supposed to be up
					if ( 	( 	$ifStats{ifOperStatus} == 2 
								and $ifStats{ifAdminStatus} == 2 
								and $interfaceTable{$intf}{ifAdminStatus} eq "up" 
						) or ( 
								$ifStats{ifOperStatus} == 1 
								and $interfaceTable{$intf}{ifOperStatus} eq "down" 
						)
					) {
						# This interface was up before but not now so recreate the interface file.
						if ($debug) { print "\tCreating an Interface file as $interfaceTable{$intf}{ifDescr} has changed ifAdminStatus.\n"; }
						# Reload the interface config won't get that one right but should get the next one right
						logMessage("runInterface, $node, $interfaceTable{$intf}{ifDescr} has changed ifAdminStatus ifOperStatus"); 
						if ( $createdone eq "false" ) {
							&createInterfaceFile($node); 
							$createdone = "true";
						}
						checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details => "interface=$interfaceTable{$intf}{ifDescr}$int_desc");
						# could also perform a notify here if you wanted to.
					}
					elsif ( $ifStats{ifOperStatus} != 1 and
							$interfaceTable{$intf}{ifAdminStatus} eq "up" 
					) {
						# ignore up/down events for ISDN type interfaces.
						if ( 	$interfaceTable{$intf}{ifDescr} !~ /$qr_ignore_up_down_ifDescr/i and
							$interfaceTable{$intf}{ifType} !~ /$qr_ignore_up_down_ifType/i
						) {
							notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details =>"interface=$interfaceTable{$intf}{ifDescr}$int_desc");
						}
					}
					else {
						# Check if interface was down before
						checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Interface Down", details => "interface=$interfaceTable{$intf}{ifDescr}$int_desc");
					}
				
					# If new ifDescr is different from old ifDescr and its a cisco router
					# run createInterfaceFile and loadInterfaceFile and everything is back to date.
					# this handles auto discovery etc on the fly. 
					## KS 1 Apr 02 - Integrating Phil Reilly's Nortel changes
					if (    $NMIS::systemTable{nodeModel} !~ /Catalyst|Accelar|BayStack/ and 
						$ifStats{ifDescr} ne $interfaceTable{$intf}{ifDescr} and 
						$ifStats{ifDescr} ne "" 
					) {
						if ($debug==2) { print "Creating an Interface file\n"; }
						# Reload the interface config won't get that one right but should get the next one right
						logMessage("runInterface, $node, ifIndex to ifDescr has Changed bad updating Interface File"); 
						&createInterfaceFile($node); 
						undef %interfaceTable;
						%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
	
						# Maybe issue and alert for reboot and possibly other bad things like config change etc
					}
				
					# RRD Database Exists so update it
					&updateRRDDB(type => "interface", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}, ifDescr => $interfaceTable{$intf}{ifDescr}, ifType => $interfaceTable{$intf}{ifType}, ifIndex => $interfaceTable{$intf}{ifIndex}, ifSpeed => $interfaceTable{$intf}{ifSpeed});
					if ( $interfaceTable{$intf}{ifType} =~ /$qr_int_stats/i ) {
						&updateRRDDB(type => "packets", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}, ifDescr => $interfaceTable{$intf}{ifDescr}, ifType => $interfaceTable{$intf}{ifType}, ifIndex => $interfaceTable{$intf}{ifIndex}, ifSpeed => $interfaceTable{$intf}{ifSpeed});
					}
					#&updateRRDDB(type => "interface", node => $node, ifDescr => $interfaceTable{$intf}{ifDescr}, ifIndex => $interfaceTable{$intf}{ifIndex});
				}
			} # create true!
		} # collect eq true
		else {
			if ($debug==5) { print "    NOT Interface:  $interfaceTable{$intf}{ifDescr}: collect=$collect, ifAdminStatus=$interfaceTable{$intf}{ifAdminStatus}, Interface Collect=$interfaceTable{$intf}{collect}\n"; }
		}
	} # FOR LOOP
	# Finished with the SNMP
	END_runInterface:
	if (defined $session) { $session->close(); }
	if ($debug) { print returnTime." Finished getting Interface stats\n"; }
} # runInterface

sub runHealth {
	my $node = shift;
	my $session;
	my $intf;
	my $message;

	undef %snmpTable;
	
	if ($debug) { print returnTime." Starting Health Stats\n"; }

	# OPEN the SNMP Session 
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	if ( not defined($session) ) { 
		warn returnTime." runHealth, Session is not Defined Goodly to $node.\n"; 
		goto END_runHealth;
	}

	# Now depending on the node type do different things.
	if ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000Sup3/ ) {
		# Get all the Cisco Health Stuff unless not doing collection
		if ($collect eq "true") {
			(	$snmpTable{avgBusy1},
				$snmpTable{avgBusy5},
				$snmpTable{MemoryUsedDRAM},
				$snmpTable{MemoryFreeDRAM},
				$snmpTable{MemoryUsedMBUF},
				$snmpTable{MemoryFreeMBUF},
				$snmpTable{MemoryUsedCLUSTER},
				$snmpTable{MemoryFreeCLUSTER},
				$snmpTable{sysTraffic},
				$snmpTable{TopChanges}
			) = $session->snmpget(  
				'cpmCPUTotal1min'.".9", 
				'cpmCPUTotal5min'.".9",
				'ciscoMemoryPoolUsed'.".1",
				'ciscoMemoryPoolFree'.".1",
				'ciscoMemoryPoolUsed'.".8",
				'ciscoMemoryPoolFree'.".8",
				'ciscoMemoryPoolUsed'.".9",
				'ciscoMemoryPoolFree'.".9",
				'sysTraffic',
				'dot1dStpTopChanges'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runHealth, $message");
				if ($debug) { print returnTime." runHealth, $message\n"; }
				goto END_runHealth;
			}

			$reach{cpu} = $snmpTable{avgBusy5}; 
						if ( $snmpTable{MemoryFreeDRAM} > 0 and $snmpTable{MemoryUsedDRAM} > 0 ) {
							$reach{mem} = ( $snmpTable{MemoryFreeDRAM} * 100 ) / ($snmpTable{MemoryUsedDRAM} + $snmpTable{MemoryFreeDRAM}) ;
						}	
						else {	
							$reach{mem} = "U";	
						}

			if ($debug) { 
				print returnTime." Health Stats Summary\n";
				print "\tCPU:1=$snmpTable{avgBusy1}, 5=$snmpTable{avgBusy5} \n\tProcMem:$snmpTable{MemoryUsedDRAM}, $snmpTable{MemoryFreeDRAM} \n\tIOMem:$snmpTable{MemoryUsedMBUF}, $snmpTable{MemoryFreeMBUF} \n\tsysTraffic=$snmpTable{sysTraffic}, TopChanges=$snmpTable{TopChanges}\n";			    
			}

			if ( &createRRDDB(type => "nodehealth", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}) ) { 
				&updateRRDDB(type => "nodehealth",node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype});
			}
		} # collect eq true
	} # nodeModel eq Catalyst6000
	elsif ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		# Get all the Cisco Health Stuff unless not doing collection
		if ($collect eq "true") {
			(	$snmpTable{avgBusy1},
				$snmpTable{avgBusy5},
				$snmpTable{MemoryUsedPROC},
				$snmpTable{MemoryFreePROC},
				$snmpTable{MemoryUsedIO},
				$snmpTable{MemoryFreeIO},
				$snmpTable{bufferElFree},
				$snmpTable{bufferElHit},
				$snmpTable{bufferFail},
				$snmpTable{ipInReceives},
				$snmpTable{ipForwDatagrams},
				$snmpTable{ipInDelivers},
				$snmpTable{ipOutNoRoutes}
			) = $session->snmpget(  
				'avgBusy1',
				'avgBusy5',
				'ciscoMemoryPoolUsed'.".1",
				'ciscoMemoryPoolFree'.".1",
				'ciscoMemoryPoolUsed'.".2",
				'ciscoMemoryPoolFree'.".2",
				'bufferElFree',
				'bufferElHit',
				'bufferFail',
				'ipInReceives',
				'ipForwDatagrams',
				'ipInDelivers',
				'ipOutNoRoutes'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runHealth, $message");
				if ($debug) { print returnTime." runHealth, $message\n"; }
				$SNMP_Simple::errmsg = "";
				goto END_runHealth;
			}
			if ($debug) { 
				print returnTime." Health Stats Summary\n";
				print "\tCPU:1=$snmpTable{avgBusy1}, 5=$snmpTable{avgBusy5}\n\tProcMem:$snmpTable{MemoryUsedPROC}, $snmpTable{MemoryFreePROC}\n\tIOMem:$snmpTable{MemoryUsedIO}, $snmpTable{MemoryFreeIO}\n\tBuffer:free=$snmpTable{bufferElFree}, hit=$snmpTable{bufferElHit}, fail=$snmpTable{bufferFail}\n\tIPIN: rec=$snmpTable{ipInReceives}, for=$snmpTable{ipForwDatagrams}, deliv=$snmpTable{ipInDelivers}, norout=$snmpTable{ipOutNoRoutes}\n";			      
			}
			
			$reach{cpu} = $snmpTable{avgBusy5}; 

			if ( $snmpTable{MemoryUsedPROC} == 0 or $snmpTable{MemoryFreePROC} == 0 ) {
				$reach{mem} = 100;
			}
			else {
				$reach{mem} = ( $snmpTable{MemoryFreePROC} * 100 ) / ($snmpTable{MemoryUsedPROC} + $snmpTable{MemoryFreePROC}) ; 
			}

			# Check if the RRD Database Exists
			if ( &createRRDDB(type => "nodehealth", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}) ) { 
				&updateRRDDB(type => "nodehealth", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype});
			}
		} # collect eq true

	} # nodeModel eq CiscoRouter
	## KS 1 Apr 02 - Integrating Phil Reilly's Nortel changes
	elsif ( $NMIS::systemTable{nodeModel} =~ /Accelar/ ) {
		# Get all the Accelar Health Stuff unless not doing collection
		if ($collect eq "true") {
			(	$snmpTable{rcSysCpuUtil},
				$snmpTable{rcSysSwitchFabricUtil},
				$snmpTable{rcSysBufferUtil},
				$snmpTable{ipInReceives},
				$snmpTable{ipForwDatagrams},
				$snmpTable{ipInDelivers},
				$snmpTable{ipOutNoRoutes}
			) = $session->snmpget(  
				'rcSysCpuUtil'.".0",
				'rcSysSwitchFabricUtil'.".0",
				'rcSysBufferUtil'.".0",
				'ipInReceives',
				'ipForwDatagrams',
				'ipInDelivers',
				'ipOutNoRoutes'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("runHealth, $message");
				if ($debug) { print returnTime." runHealth, $message\n"; }
				$SNMP_Simple::errmsg = "";
				goto END_runHealth;
			}
			if ($debug) { 
				print returnTime." Health Stats Summary\n";

				print "\tCPU=$snmpTable{rcSysCpuUtil}, SFabric=$snmpTable{rcSysSwitchFabricUtil}\n\tBuffer=$snmpTable{rcSysBufferUtil},IPIN: rec=$snmpTable{ipInReceives}, for=$snmpTable{ipForwDatagrams}, deliv=$snmpTable{ipInDelivers}, norout=$snmpTable{ipOutNoRoutes}\n";			      
			}
			
			$reach{cpu} = $snmpTable{rcSysCpuUtil}; 

			# Check if the RRD Database Exists
			if ( &createRRDDB(type => "nodehealth", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}) ) { 
				&updateRRDDB(type => "nodehealth", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype});
			}
		} # collect eq true
	} # nodeModel eq Accelar
	
	# Finished with the SNMP
	END_runHealth:
	if (defined $session) { $session->close(); }
} # end runHealth

sub thresholdProcess {
	## KS 16/4/01 - used by runThreshold.
	my %args = @_;
	if ( $args{value} =~ /^\d+$|^\d+\.\d+$/ ) {
		if ($debug) { print returnTime." Threshold: $args{event} level=$args{level} value=$args{value}\n"; }
		if ( $args{value} !~ /NaN/i and $args{value} != 0 ) {
			if ( $args{level} =~ /Normal/ ) { 
				checkEvent(node => $args{node}, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $args{event}, level => $args{level}, details => "Threshold=$args{value}");
			}
			else {
				notify(node => $args{node}, role => $args{role}, type => $args{type}, event => $args{event}, level => $args{level}, details => "Threshold=$args{value}");
			}
		}
	}
}

## KS 1 April 02 - Implemented Thresholds Policy
sub runThreshold {
	my $node = shift;

	my %threshold;
	my $reportStats;
	my @tmparray;
	my @tmpsplit;
	my $level;
	my $event;
	my $operAvailability;
	my $totalUtil;
	my $inputUtil;
	my $outputUtil;
	my $reportStats;
	my @tmparray;
	my $intf;
	my %interfaceTable;

	#available
	#response
	#reachable

	#util
	#cpu
	#mem
	#nonucast	

	++$nodecount;

	if ($debug) { 
		print "\n";
		print returnTime." Starting Thresholding node=$node collect=$NMIS::nodeTable{$node}{collect}\n"; 
	}
	
	loadSystemFile($node);

	my %thresholds = loadCSV($NMIS::config{Thresholds_Table},$NMIS::config{Thresholds_Key},"\t");

	# Get the device health thresholds
	$reportStats = summaryStats($node,"health","-15 minutes",time);
	if ( $reportStats != 1 and $reportStats ne "" ) {
		@tmparray = split " ","@$reportStats";
		for ( $index = 0; $index <= $#tmparray; ++$index ) {
			@tmpsplit = split "=",@tmparray[$index];
			$threshold{$tmpsplit[0]} = $tmpsplit[1];
		}
	}
	
	# Now we have a nicely populated threshold table, lets compare them and issue some alerts
	$level = thresholdPolicy(threshold => "response", value =>$threshold{response}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
	$event = "Proactive Response Time Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{response});

	$level = thresholdPolicy(threshold => "reachable", value =>$threshold{reachable}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
	$event = "Proactive Reachability Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{reachable});

	$level = thresholdPolicy(threshold => "available", value =>$threshold{available}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
	$event = "Proactive Interface Availability Threshold";
	thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{available});

	# there is extra stuff to threshold!
	# Will need to add something to summary stats to threshold switch stats
	if ( 	$NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS|CiscoATM/ 
		and $NMIS::nodeTable{$node}{collect} eq "true"
	) {
		$reportStats = summaryStats($node,"cpu","-15 minutes",time);
		if ( $reportStats != 1 and $reportStats ne "" ) {
			@tmparray = split " ","@$reportStats";
			for ( $index = 0; $index <= $#tmparray; ++$index ) {
				@tmpsplit = split "=",@tmparray[$index];
				$threshold{$tmpsplit[0]} = $tmpsplit[1];
			}
	
			$level = thresholdPolicy(threshold => "cpu", value =>$threshold{avgBusy5min}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
			$event = "Proactive CPU Threshold";
			thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{avgBusy5min});
	
			$level = thresholdPolicy(threshold => "mem", value =>$threshold{ProcMemFree}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
			$event = "Proactive Memory Threshold";
			thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{ProcMemFree});
		}
		else {
			if ($debug) { print returnTime." Problem with Health Stats on $node.\n"; }
		}
	}
	## KS 1 Apr 02 - Integrating Phil Reilly's Nortel changes
	elsif ( 	$NMIS::systemTable{nodeModel} =~ /Accelar/ 
		and $NMIS::nodeTable{$node}{collect} eq "true"
	) {
		$reportStats = summaryStats($node,"acpu","-15 minutes",time);
		if ( $reportStats != 1 and $reportStats ne "" ) {
			@tmparray = split " ","@$reportStats";
			for ( $index = 0; $index <= $#tmparray; ++$index ) {
				@tmpsplit = split "=",@tmparray[$index];
				$threshold{$tmpsplit[0]} = $tmpsplit[1];
			}
	
			$level = thresholdPolicy(threshold => "cpu", value =>$threshold{avgBusy5min}, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, thresholds => \%thresholds);
			$event = "Proactive CPU Threshold";
			thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $threshold{rcSysCpuUtil});
	
		}
		else {
			if ($debug) { print returnTime." Problem with Health Stats on $node.\n"; }
		}
	}
	
	# Threshold interface utilisation!  Easy to do, just take an hour or so.
	# hmmmm I was wrong, about 20mins plus some testing. (I love reusable code)
	if ( 	$NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000|CiscoRouter|CatalystIOS|CiscoATM|generic|SunSolaris|Win2000|Accelar|BayStack/
		and $NMIS::nodeTable{$node}{collect} eq "true"
	) {
		%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","ifIndex","\t");
		# Extract the interface statics and summaries for display in a second.
		foreach $intf (keys %interfaceTable) {
			# Don't do any stats cause the interface is not one we collect
			if ( $interfaceTable{$intf}{collect} eq "true" ) { 
				# Get the link availability from the local node!!!
				$reportStats = summaryStats($node,"util","-15 minutes",time,$interfaceTable{$intf}{ifDescr},$interfaceTable{$intf}{ifSpeed});
				if ( $reportStats != 1 and $reportStats ne "" ) {
					@tmparray = split " ","@$reportStats";
					$operAvailability = $tmparray[0];
					$inputUtil = $tmparray[1];
					$outputUtil = $tmparray[2];
					$totalUtil = $tmparray[3];
					$operAvailability =~ s/(.*)=(.*)/$2/;
					$inputUtil =~ s/(.*)=(.*)/$2/;
					$outputUtil =~ s/(.*)=(.*)/$2/;
					$totalUtil =~ s/(.*)=(.*)/$2/;
				}

				# Interface Input Utilisation
				$level = thresholdPolicy(threshold => "util", value => $inputUtil, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, interface => $interfaceTable{$intf}{ifDescr}, thresholds => \%thresholds);
				$event = "Proactive Interface Input Utilisation Threshold $interfaceTable{$intf}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $inputUtil);

				# Interface Output Utilisation
				$level = thresholdPolicy(threshold => "util", value => $outputUtil, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, interface => $interfaceTable{$intf}{ifDescr}, thresholds => \%thresholds);
				$event = "Proactive Interface Output Utilisation Threshold $interfaceTable{$intf}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $outputUtil);

				# Interface Availability
				$level = thresholdPolicy(threshold => "int_avail", value => $operAvailability, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, interface => $interfaceTable{$intf}{ifDescr}, thresholds => \%thresholds);
				$event = "Proactive Availability Threshold Interface $interfaceTable{$intf}{ifDescr}";
				thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $operAvailability);
				
				# Non Unicast traffic (broadcast/multicast)
				if ( $interfaceTable{$intf}{ifType} =~ /$qr_int_stats/i ) {
					$reportStats = summaryStats($node,"pkts","-15 minutes",time,$interfaceTable{$intf}{ifDescr},$interfaceTable{$intf}{ifSpeed});
					my $ifInUcastPkts;
					my $ifInNUcastPkts;
					my $ifInDiscards;
					my $ifInErrors;
					my $ifOutUcastPkts;
					my $ifOutNUcastPkts;
					my $ifOutDiscards;
					my $ifOutErrors;
					if ( $reportStats != 1 and $reportStats ne "" ) {
						@tmparray = split " ","@$reportStats";
						$ifInUcastPkts = $tmparray[0];
						$ifInNUcastPkts = $tmparray[1];
						$ifInDiscards = $tmparray[2];
						$ifInErrors = $tmparray[3];
						$ifOutUcastPkts = $tmparray[4];
						$ifOutNUcastPkts = $tmparray[5];
						$ifOutDiscards = $tmparray[6];
						$ifOutErrors = $tmparray[7];

						$ifInUcastPkts =~ s/(.*)=(.*)/$2/;
						$ifInNUcastPkts =~ s/(.*)=(.*)/$2/;
						$ifInDiscards =~ s/(.*)=(.*)/$2/;
						$ifInErrors =~ s/(.*)=(.*)/$2/;
						$ifOutUcastPkts =~ s/(.*)=(.*)/$2/;
						$ifOutNUcastPkts =~ s/(.*)=(.*)/$2/;
						$ifOutDiscards =~ s/(.*)=(.*)/$2/;
						$ifOutErrors =~ s/(.*)=(.*)/$2/;
					}
					$level = thresholdPolicy(threshold => "nonucast", value => $ifInNUcastPkts, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, interface => $interfaceTable{$intf}{ifDescr}, thresholds => \%thresholds);
					$event = "Proactive Interface Input NonUnicast Threshold $interfaceTable{$intf}{ifDescr}";
					thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $ifInNUcastPkts);

					$level = thresholdPolicy(threshold => "nonucast", value => $ifOutNUcastPkts, node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, interface => $interfaceTable{$intf}{ifDescr}, thresholds => \%thresholds);
					$event = "Proactive Interface Output NonUnicast Threshold $interfaceTable{$intf}{ifDescr}";
					thresholdProcess(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => $event, level => $level, value => $ifOutNUcastPkts);
				}
			}
		} # FOR LOOP
	}
} # end runThreshold

### AS 6 June 2002 - Added escalate 0, to allow fast escalation and to implement 
### consistent policies for notification.  This also helps to get rid of flapping
### things, ie if escalate0 = 5 then an interface goes down, no alert sent, next 
### poll interface goes up and event cancelled!  Downside is a little longer before
### receiving first notification, so it depends on what the support SLA is.
sub runEscalate {

	my $outage_time;
	my $planned_outage;
	my $event_hash;
	my $send = "false";
	my %location_data;
	my $target;
	my $time;
	my $escalate;
	my $event_age;
	my %contact_table;
	my @x;
	my $i;
	
	if ($debug) { print "\n".returnTime." Running escalate...\n"; }
	
	%contact_table = loadCSV($NMIS::config{Contacts_Table},$NMIS::config{Contacts_Key},"\t");

	# Load the event table into the hash
	loadEventState;
	
	foreach $event_hash ( sort(keys(%NMIS::eventTable)) )  {
	    $outage_time = time - $NMIS::eventTable{$event_hash}{startdate};
	    
	    # The event must not be part of a planned outage
	    if ( 	outageCheck($NMIS::eventTable{$event_hash}{node},time) ne "true" 
	    		and $NMIS::eventTable{$event_hash}{ack} eq "false"
	    ) {
			$escalate = $NMIS::eventTable{$event_hash}{escalate};
			# now depending on the event escalate the event up a level or so depending on how long it has been active
			# now would be the time to notify as to the event. node down every 15 minutes, interface down every 4 hours?
			# maybe a descreasing run 15,30,60,2,4,etc
			# proactive events would be escalated daily
			# when escalation hits 10 they could auto delete?
			# core, distrib and access could escalate at different rates.

			if ($debug>3) { print "Pre Escalation : $NMIS::eventTable{$event_hash}{node} Event $NMIS::eventTable{$event_hash}{event} is $outage_time seconds old escalation is $NMIS::eventTable{$event_hash}{escalate}\n"; }
			if ( $NMIS::eventTable{$event_hash}{event} eq "Node Down" ) {
				if (    $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 8 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 4 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
				elsif ( $outage_time >= $NMIS::config{escalate0} * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 0; $send = "true"; }
			}
			elsif ( $NMIS::eventTable{$event_hash}{event} eq "Interface Down" ) {
				if (    $outage_time >= 24 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 6 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
				elsif ( $outage_time >= $NMIS::config{escalate0} * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 0; $send = "true"; }
			}
			# default escalation for "DOWN events
			elsif ( $NMIS::eventTable{$event_hash}{event} =~ /Down/ ) {
				if (    $outage_time >= 24 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 6 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
				elsif ( $outage_time >= $NMIS::config{escalate0} * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 0; $send = "true"; }
			}
			elsif ( $NMIS::eventTable{$event_hash}{event} =~ /proactive/i ) {
				if (    $outage_time >= 24 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 10; $send = "true"; }
				elsif ( $outage_time >= 12 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 9; $send = "true"; }
				elsif ( $outage_time >= 6 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 8; $send = "true"; }
				elsif ( $outage_time >= 3 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 7; $send = "true"; }
				elsif ( $outage_time >= 2 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 6; $send = "true"; }
				elsif ( $outage_time >= 1 * 60 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 5; $send = "true"; }
				elsif ( $outage_time >= 50 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 4; $send = "true"; }
				elsif ( $outage_time >= 40 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 3; $send = "true"; }
				elsif ( $outage_time >= 30 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 2; $send = "true"; }
				elsif ( $outage_time >= 15 * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 1; $send = "true"; }
				elsif ( $outage_time >= $NMIS::config{escalate0} * 60 ) { $NMIS::eventTable{$event_hash}{escalate} = 0; $send = "true"; }
			}
		
			if ($debug>3) { print "Post Escalation: $NMIS::eventTable{$event_hash}{node} Event $NMIS::eventTable{$event_hash}{event} is $outage_time seconds old escalation is $NMIS::eventTable{$event_hash}{escalate}\n"; }
		}
		# send a new email message as the escalation again.
		if ( 	$send eq "true" 
				and $escalate != $NMIS::eventTable{$event_hash}{escalate} 
				and $NMIS::eventTable{$event_hash}{event} !~ /Proactive/
		) {
			loadSystemFile($NMIS::eventTable{$event_hash}{node});
			
			$event_age = convertSecsHours(time - $NMIS::eventTable{$event_hash}{startdate});
			$time = &returnDateStamp;
			
			$target = lc(escalationPolicy( group => $NMIS::nodeTable{$NMIS::eventTable{$event_hash}{node}}{group}, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, level => $NMIS::eventTable{$event_hash}{escalate} ));
			
			if ( $target ne "" ) {
				# Now we have a target, just in case, check for multiple contacts
				@x = split(":",$target);
				$target = "";
				if ( $x[0] eq "email" ) {
					for ( $i = 1; $i <= $#x; ++$i ) {
						if ( $contact_table{$x[$i]}{Email} ne "" ) {
							$target = $target." ".$contact_table{$x[$i]}{Email};
						}
						#print "\tx=$x[$i] email=$contact_table{$x[$i]}{Email}\n"
					}
					if ( $target eq "" ) { $target = $contact_table{default}{Email}; }
					#print returnTime." runEscalate x=@x hash=$contact_table{$x[1]}{Email} target=$target\n";
							
					sendEmail(
						to => $target, 
						subject => "$time Escalation $NMIS::eventTable{$event_hash}{escalate} $event_age $NMIS::eventTable{$event_hash}{node} $NMIS::eventTable{$event_hash}{event_level} $NMIS::eventTable{$event_hash}{event} $NMIS::eventTable{$event_hash}{details}", 
						body => "$time Escalation $NMIS::eventTable{$event_hash}{escalate} $event_age $NMIS::eventTable{$event_hash}{node} $NMIS::eventTable{$event_hash}{event_level} $NMIS::eventTable{$event_hash}{event} $NMIS::eventTable{$event_hash}{details}\nhttp://$NMIS::config{nmis_host}$NMIS::config{nmis}?type=event&node=$NMIS::eventTable{$event_hash}{node}",
						from => $NMIS::config{mail_from}, 
						server => $NMIS::config{mail_server}, 
						domain => $NMIS::config{mail_domain}, 
						debug => $NMIS::debug
					);
				}
				else {
					print STDERR "\tERROR runEscalate problem with escalation target @x\n";
				}
			}
			if ($debug>4) { print "\tESCALATION NOTIFICATION node=$NMIS::eventTable{$event_hash}{node} target=$target level=$NMIS::eventTable{$event_hash}{event_level} event=$NMIS::eventTable{$event_hash}{event} details=$NMIS::eventTable{$event_hash}{details} group=$NMIS::nodeTable{$NMIS::eventTable{$event_hash}{node}}{group}\n"; }
		}
		$send = "false";
	} # foreach $event_hash
	writeEventState;
} # end runEscalate

sub runUpdate {
	# assuming that nothing with collect eq to false has been sent here.
	my $node = shift;
	my $pingresult = shift;
	
	if ($debug) { print returnTime." Running an Update for node=$node, type=$NMIS::systemTable{nodeType}\n"; }
		
	# Update supported by all devices
	&createSystemFile($node, $pingresult);
	&createInterfaceFile($node);
}

## KS 30 Nov 2001 - to fit with Eric's imlnetnz@yahoo.co.nz change for better Pinging.
sub runPing {
	my $node = shift;
	if ($debug) { print returnTime." Starting Pinging with timeout=$NMIS::config{ping_timeout} retries=$NMIS::config{ping_retries}\n"; }
	my $ping = ping(node => $node, timeout => $NMIS::config{ping_timeout}, count => $NMIS::config{ping_retries}, packet => $NMIS::config{ping_packet}, debug => $debug );
	if ( $ping != 100 ) {
		# Device is down
		$ping = 0; 
		if ($debug) { print returnTime." Pinging Failed $node $NMIS::systemTable{roleType} $NMIS::systemTable{nodeType}\n"; }
		notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Down");
	} else {
		# Device is UP!
		checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Down");
	}
	if ($debug) { print " result=$ping\n"; }
	return($ping);	
}

sub runReachability {
	my $node = shift;
	my $pingresult = shift;
	
	my $cpuWeight;
	my $memWeight;
	my $responseWeight;
	my $interfaceWeight;
	my %interfaceTable;
	my $intf;
	my $inputUtil;
	my $outputUtil;
	my $totalUtil;
	my $index;
	my $reportStats;
	my @tmparray;
	my @tmpsplit;
	my %util;
	my $intcount;
	my $intsummary;
	my $intWeight;
	
	if ($debug) { print returnTime." Running Reachability node=$node type=$NMIS::nodeTable{$node}{devicetype}\n"; }
	
	# Things which don't do collect get 100 for availability
	if ( $reach{availability} eq "" and $collect eq "false" ) { 
		$reach{availability} = "100"; 
	}
	elsif ( $reach{availability} eq "" ) { $reach{availability} = "U"; }
	
	# Health should actually reflect a combination of these values
	# ie if response time is high health should be decremented.
	if ( $pingresult == 100 and $snmpresult == 100 ) {
		$reach{reachability} = 100;
		if ( $reach{operCount} > 0 ) {
			$reach{availability} = $reach{operStatus} / $reach{operCount}; 
		}
		$reach{responsetime} = responseTime(node => "$node", count => 5, debug => "$debug");
		($reach{responsetime},$responseWeight) = weightResponseTime($reach{responsetime});
		
		if ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} !~ /generic|SunSolaris|Win2000/i ) {
			if    ( $reach{cpu} <= 10 ) { $cpuWeight = 100; }
			elsif ( $reach{cpu} <= 20 ) { $cpuWeight = 90; }
			elsif ( $reach{cpu} <= 30 ) { $cpuWeight = 80; }
			elsif ( $reach{cpu} <= 40 ) { $cpuWeight = 70; }
			elsif ( $reach{cpu} <= 50 ) { $cpuWeight = 60; }
			elsif ( $reach{cpu} <= 60 ) { $cpuWeight = 50; }
			elsif ( $reach{cpu} <= 70 ) { $cpuWeight = 40; }
			elsif ( $reach{cpu} <= 80 ) { $cpuWeight = 30; }
			elsif ( $reach{cpu} <= 90 ) { $cpuWeight = 20; }
			elsif ( $reach{cpu} <= 100 ) { $cpuWeight = 10; }
			
			if    ( $reach{mem} >= 40 ) { $memWeight = 100; }
			elsif ( $reach{mem} >= 35 ) { $memWeight = 90; }
			elsif ( $reach{mem} >= 30 ) { $memWeight = 80; }
			elsif ( $reach{mem} >= 25 ) { $memWeight = 70; }
			elsif ( $reach{mem} >= 20 ) { $memWeight = 60; }
			elsif ( $reach{mem} >= 15 ) { $memWeight = 50; }
			elsif ( $reach{mem} >= 10 ) { $memWeight = 40; }
			elsif ( $reach{mem} >= 5 )  { $memWeight = 25; }
			elsif ( $reach{mem} >= 0 )  { $memWeight = 0; }
		}
		elsif ( $NMIS::nodeTable{$node}{collect} eq "true" and $NMIS::systemTable{nodeModel} eq "generic" ) {
			$cpuWeight = 100;
			$memWeight = 100;
		}
		else {
			$cpuWeight = 100;
			$memWeight = 100;
			$reach{availability} = 100;
		}
		
		## KS 13/4/01 - Added little fix for when no interfaces are collected.
		if ( $reach{availability} !~ /\d+/ ) {
			$reach{availability} = "100";
		}
		
		## KS 1/5/01 - Makes CatalystIOS memory health weighting always 100.
		## PR 11/11/01 - Add Baystack and Accelar
		if ( $NMIS::systemTable{nodeModel} =~ /CatalystIOS|Accelar|BayStack/i ) {
			$memWeight = 100;
		}
		
		if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
			if ($debug) { print "\tGetting Interface Utilisation Health\n"; }
			$intcount = 0;
			$intsummary = 0;
			%interfaceTable = &loadCSV("$NMIS::config{'<nmis_var>'}/$node-interface.dat","interface","\t");
			foreach $intf (keys %interfaceTable) {
				# Don't do any stats cause the interface is not one we collect
				if ( $interfaceTable{$intf}{collect} eq "true" ) { 
				# Get the link availability from the local node!!!
					$reportStats = summaryStats($node,"util","-15 minutes",time,$interfaceTable{$intf}{ifDescr},$interfaceTable{$intf}{ifSpeed});
					if ( $reportStats != 1 and $reportStats ne "" ) {
						@tmparray = split " ","@$reportStats";
						for ( $index = 0; $index <= $#tmparray; ++$index ) {
							@tmpsplit = split "=",@tmparray[$index];
							$util{$tmpsplit[0]} = $tmpsplit[1];
						}
					}
					$intsummary = $intsummary + ( 100 - $util{inputUtil} ) + ( 100 - $util{outputUtil} );
					++$intcount;
					#print "in=$util{inputUtil} out=$util{outputUtil} intsumm=$intsummary count=$intcount\n";
				}
			} # FOR LOOP
			if ( $intsummary != 0 ) {
				$intWeight = sprintf( "%.2f", $intsummary / ( $intcount * 2 ));
			}
			else {
				$intWeight = "NaN"
			}
		}
		else {
			$intWeight = 100;	
		}
		
		# if the interfaces are unhealthy and lost stats, whack a 100 in there
		if ( $intWeight eq "NaN" or $intWeight > 100 ) { $intWeight = 100; }
		
		# Would be cool to collect some interface utilisation bits here.
		# Maybe thresholds are the best way to handle that though.  That
		# would pickup the peaks better.
		
		# Health is made up of a weighted values:
		## KS 16 Mar 02, implemented weights in nmis.conf
		$reach{health} = 	($reach{reachability} * $NMIS::config{weight_reachability}) + 
							($intWeight * $NMIS::config{weight_int}) +
							($responseWeight * $NMIS::config{weight_response}) + 
							($reach{availability} * $NMIS::config{weight_availability}) + 
							($cpuWeight * $NMIS::config{weight_cpu}) + 
							($memWeight * $NMIS::config{weight_mem})
							;
	}
	elsif ( $NMIS::nodeTable{$node}{collect} eq "false"
		and $pingresult == 100 
	) {
		$reach{reachability} = 100; 
		$reach{availability} = 100; 
		$reach{responsetime} = responseTime(node => "$node", count => 5, debug => "$debug");
		($reach{responsetime},$responseWeight) = weightResponseTime($reach{responsetime});
		$reach{health} = ($reach{reachability} * 0.9) + ( $responseWeight * 0.1);
	}
	elsif ( $pingresult == 0 and NMIS::outageCheck($node,time) eq "true" ) {
		if ($debug) { print returnTime." Planned Outage Exists\n"; }
		$reach{reachability} = "U"; 
		$reach{availability} = "U"; 
		$reach{responsetime} = "U"; 
		$reach{health} = "U"; 
	} 
	elsif ( $pingresult == 100 and $snmpresult == 0 ) {
		$reach{reachability} = 100; 
		$reach{availability} = "U"; 
		$reach{responsetime} = responseTime(node => "$node", count => 5, debug => "$debug"); 
		$reach{health} = "U"; 
	}
	else {
		$reach{reachability} = 0; 
		$reach{availability} = "U"; 
		$reach{responsetime} = "U"; 
		$reach{health} = 0; 
	}
	
	if ($debug) { print returnTime." Health: collect=$NMIS::nodeTable{$node}{collect} ping=$pingresult reach=$reach{reachability} avail=$reach{availability} rt=$reach{responsetime} cpu=$cpuWeight mem=$memWeight int=$intWeight health=$reach{health}\n"; }
	
	if ($debug>2) { print returnTime." Doing Reachability database stuff!\n"; }
	if ( &createRRDDB(type => "reach", node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype}) ) { 
		&updateRRDDB(type => "reach",node => $node, nodeType => $NMIS::nodeTable{$node}{devicetype});
	}
}

sub createSystemFile {
	my $node = shift;
	my $pingresult = shift;
	
	undef %NMIS::systemTable;
	if ( $NMIS::nodeTable{$node}{collect} eq "true" and $pingresult == 100 ) {
		&getNodeInfo($node);
	}
	else {
		$writeSystem = "true"	
	}
	#This makes the nodes file overwrite the getNodeInfo bits.
	$NMIS::systemTable{netType} = $NMIS::nodeTable{$node}{net};
	$NMIS::systemTable{roleType} = $NMIS::nodeTable{$node}{role};
	$NMIS::systemTable{nodeType} = $NMIS::nodeTable{$node}{devicetype};
	
	&writeSystemFile($node);
}

sub getNodeInfo {
	my $node = shift;
	my $message;
	
	my $session;
	my %enterprise = loadCSV($NMIS::config{Enterprise_Table},$NMIS::config{Enterprise_Key},"\t");
	my %models = loadCSV($NMIS::config{Model_Table},$NMIS::config{Model_Key},"\t");
	my $OID;
	my $oid_key;
	my @x;
	
	if ($debug) { print returnTime." Getting Node Info using SNMP Ver: $NMIS::systemTable{snmpVer}\n"; }
	
	# only test for SNMPv2 support the first time the device is discovered
	# delete the system file or edit the snmpVer attribute to rediscover SNMP support.
	if ( $NMIS::systemTable{snmpVer} eq "" ) {
		# now open SNMPv2 and see if the device talks that!
		if ($debug) { print returnTime." Testing for SNMPv2 support\n"; }
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
		if ( not defined($session) ) { 
			warn returnTime." getNodeInfo, Session is not Defined Goodly to $node.\n"; 
			goto END_getNodeInfo;
		}

		(	$NMIS::systemTable{sysObjectID},
			$NMIS::systemTable{sysUpTime}
		) = $session->snmpget (
			'sysObjectID', 
			'sysUpTime'
		);		
		if ( $SNMP_Simple::errmsg !~ /No answer from/ ) {
			$NMIS::systemTable{snmpVer} = "SNMPv2";
			if (defined $session) { $session->close(); }
		}
		else {
			if (defined $session) { $session->close(); }
			if ($debug) { print "\tSNMPv2 doesn't work. errmsg=$SNMP_Simple::errmsg\n"; }
			$SNMP_Simple::errmsg = "";
			if ($debug) { print returnTime." Testing for SNMPv1 support\n"; }
			($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
			if ( not defined($session) ) { 
				warn returnTime." getNodeInfo, Session is not Defined Goodly to $node.\n"; 
				goto END_getNodeInfo;
			}
			(	$NMIS::systemTable{sysObjectID},
				$NMIS::systemTable{sysUpTime}
			) = $session->snmpget (
				'sysObjectID', 
				'sysUpTime'
			);
			
			if ( $SNMP_Simple::errmsg !~ /No answer from/ ) {
				if (defined $session) { $session->close(); }
				$NMIS::systemTable{snmpVer} = "SNMPv1";
			}
			else {
				if ($debug) { print "\tSNMPv1 doesn't work. errmsg=$SNMP_Simple::errmsg\n"; }
				logMessage("getNodeInfo, $node, SNMP Error. community or SNMP agent problem. errmsg=$SNMP_Simple::errmsg");
				$SNMP_Simple::errmsg = "";
				$NMIS::systemTable{snmpVer} = "none";
				goto END_getNodeInfo;
			}
		}
		if ($debug) { print returnTime." $node supports $NMIS::systemTable{snmpVer} version.\n"; }
	}

	if ($debug) { print returnTime." getNodeInfo opening SNMP Ver: $NMIS::systemTable{snmpVer}\n"; }
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	elsif ( $NMIS::systemTable{snmpVer} eq "SNMPv1" ) {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	if ( not defined($session) ) { 
		$message = "Session is not Defined Goodly to $node.\n"; 
		logMessage("getNodeInfo, $message");
		if ($debug) { print returnTime." getNodeInfo, $message\n"; }
		goto END_getNodeInfo;
	}
	
	(	$NMIS::systemTable{sysDescr},
		$NMIS::systemTable{sysObjectID},
		$NMIS::systemTable{sysUpTime},
		$NMIS::systemTable{sysContact},
		$NMIS::systemTable{sysName},
		$NMIS::systemTable{sysLocation},
		$NMIS::systemTable{sysServices},
		$NMIS::systemTable{ifNumber}
	)
	= $session->snmpget(
		'sysDescr', 
		'sysObjectID', 
		'sysUpTime', 
		'sysContact', 
		'sysName', 
		'sysLocation', 
		'sysServices', 
		'ifNumber'
	);
	if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
		$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
		$SNMP_Simple::errmsg = "";
		logMessage("getNodeInfo, $message");
		if ($debug) { print returnTime." getNodeInfo, $message\n"; }
		goto END_getNodeInfo;
	}
	elsif ( $NMIS::systemTable{sysObjectID} =~ /unprintable BER type/ 
			or $NMIS::systemTable{sysUpTime} =~ /unprintable BER type/ 
			or $NMIS::systemTable{ifNumber} =~ /unprintable BER type/
	) {
		$snmpresult = 0;
		logMessage("getNodeInfo, $node, SNMP ERROR sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
		if ($debug)  { print returnTime." $node SNMP Problem sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
		$writeSystem = "true";
		goto END_getNodeInfo;
	}
	elsif ( $NMIS::systemTable{sysObjectID} eq "" 
			and $NMIS::systemTable{sysUpTime} eq "" 
			and $NMIS::systemTable{ifNumber} eq "" 
	) {
		$snmpresult = 0;
		logMessage("getNodeInfo, $node, SNMP elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
		if ($debug)  { print returnTime." $node elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
		goto END_getNodeInfo;
	}
	
	## KS 1/4/01 - Added this for to support default contacts and locations.
	if ( $NMIS::systemTable{sysContact} eq "" ) { $NMIS::systemTable{sysContact} = "default"; }
	if ( $NMIS::systemTable{sysLocation} eq "" ) { $NMIS::systemTable{sysLocation} = "default"; }

	# Only continue processing if at least a couple of entries are valid.
	if (    $NMIS::systemTable{sysDescr} ne "" and 
		$NMIS::systemTable{sysObjectID} ne "" and 
		$NMIS::systemTable{sysServices} ne ""
	) {	      
		$writeSystem = "true";
		
		$NMIS::systemTable{sysObjectName} = SNMP_MIB::oid2name($NMIS::systemTable{sysObjectID});

		# Decide on vendor name.
		@x = split(/\./,$NMIS::systemTable{sysObjectID});
		$OID = $x[6];
		if ( $enterprise{$OID}{Enterprise} ne "" ) {
			$NMIS::systemTable{nodeVendor} = $enterprise{$OID}{Enterprise};
		}
		else { $NMIS::systemTable{nodeVendor} =  "Unknown"; }
		
		if ($debug) { print returnTime." $node vendor=$NMIS::systemTable{nodeVendor} VendorID=$OID sysObjectName=$NMIS::systemTable{sysObjectName} sysObjectID=$NMIS::systemTable{sysObjectID}\n"; }

		# Decide on device Capabilities
		if ( $NMIS::systemTable{sysServices} == 2 ) { 
			$NMIS::systemTable{nodeType} = "switch"; 
			$NMIS::systemTable{netType} = "lan";
		}
		elsif ( $NMIS::systemTable{sysServices} == 4 or 
			$NMIS::systemTable{sysServices} == 6
		) { 
			$NMIS::systemTable{nodeType} = "router"; 
			$NMIS::systemTable{netType} = "lan";
		}				      
		elsif ( $NMIS::systemTable{sysServices} == 78 ) { 
			$NMIS::systemTable{nodeType} = "router"; 
			$NMIS::systemTable{netType} = "wan";
		}
		elsif ( $NMIS::systemTable{sysServices} == 72 or
			$NMIS::systemTable{sysServices} == 76
		) { 
			$NMIS::systemTable{nodeType} = "server"; 
			$NMIS::systemTable{netType} = "lan";
		}
		else { $NMIS::systemTable{nodeType} =  $NMIS::systemTable{sysServices}; }

		if ($debug) { print returnTime." $node sysServices=$NMIS::systemTable{sysServices}\n"; }

		my $match = 0;
		if ($debug>1) { print returnTime." $node sysDescr=$NMIS::systemTable{sysDescr}\n"; }
		foreach my $model (sort { $a <=> $b } keys %models) {
			if ($debug>1) { print "  key=$model pattern=$models{$model}{sysDescr}\n"; }
			if ( ! $match 
				and $NMIS::systemTable{nodeVendor} eq $models{$model}{nodeVendor} 
				and $NMIS::systemTable{sysDescr} =~ /$models{$model}{sysDescr}/
			) {
				if ($debug) { print returnTime." $node Found a Model, $models{$model}{nodeModel}\n"; }
				if ($debug>1) { print "  pattern=$models{$model}{sysDescr} sysDescr=$NMIS::systemTable{sysDescr}\n"; }
				$match = 1;
				$NMIS::systemTable{nodeModel} = $models{$model}{nodeModel};
				$NMIS::systemTable{nodeType} = $models{$model}{nodeType};
				$NMIS::systemTable{netType} = $models{$model}{netType};
				$NMIS::systemTable{supported} = $models{$model}{supported};
				last;
			}
		}
		if ( ! $match ) {
			$NMIS::systemTable{nodeModel} = $models{default}{nodeModel};
			#$NMIS::systemTable{nodeType} = $models{default}{nodeType};
			#$NMIS::systemTable{netType} = $models{default}{netType};
			$NMIS::systemTable{supported} = $models{default}{supported};
		}

		# if the device is a cisco router do collect some extra system info
		if (    ( $NMIS::systemTable{nodeModel} eq "CiscoRouter" ) &&
			( $NMIS::systemTable{nodeVendor} eq "Cisco Systems" )
		){
			if ($debug)  { print returnTime." Getting Cisco Router Serial Number\n"; }
			( 
				$NMIS::systemTable{chassisVer},
				$NMIS::systemTable{serialNum},
				$NMIS::systemTable{processorRam}
			) = $session->snmpget( 
				'chassisVersion',
				'chassisId',
				'processorRam'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("getNodeInfo, $message");
				if ($debug) { print returnTime." getNodeInfo, $message\n"; }
				goto END_getNodeInfo;
			}
		}
		elsif ( ( $NMIS::systemTable{nodeType} eq "switch" ) &&
			( $NMIS::systemTable{nodeVendor} eq "Cisco Systems" )
		){
			if ($debug)  { print returnTime." Getting Cisco Switch Serial Number\n"; }
			( 
				$NMIS::systemTable{serialNum},
				$NMIS::systemTable{sysTrafficPeak},
				$NMIS::systemTable{sysTrafficPeakTime}
			) = $session->snmpget( 
				'chassisSerialNumberString',
				'sysTrafficPeak',
				'sysTrafficPeakTime'
			);
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("getNodeInfo, $message");
				if ($debug) { print returnTime." getNodeInfo, $message\n"; }
				goto END_getNodeInfo;
			}
		}
	} # must have SNMP eq something.
	else {
		$writeSystem = "false";
	}
	
	# Finished with the SNMP
	END_getNodeInfo:
	if (defined $session) { $session->close(); }
} # end getNodeInfo

# Put all the good SNMP info into a file for caching and also test SNMP for use.
sub updateUptime {
	my $node = shift;
	my $pingresult = shift;
	my $newUpTime;
	my $oldUpTime;
	my $session;

    if ($debug)  { print returnTime." Updating UPTIME Info\n"; }
	
	# OPEN the SNMP Session 
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	if ( not defined($session) ) { 
		warn returnTime." updateUptime, Session is not Defined Goodly to $node.\n"; 
		goto END_updateUptime;
	}
	else {
		(	$snmpTable{sysObjectID},
			$snmpTable{sysUpTime},
			$snmpTable{ifNumber}
		) = $session->snmpget (
			'sysObjectID', 
			'sysUpTime', 
			'ifNumber'
		);
		if ($debug)  { print "\tsysObjectID=$snmpTable{sysObjectID}\tifNumber=$snmpTable{ifNumber}\tsysUpTime=$snmpTable{sysUpTime}\n"; }
	
		# Check if SNMP returned anything at all.
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$snmpresult = 0;
			if ($debug)  { print "\t$node, SNMP ERROR, errmsg=$SNMP_Simple::errmsg\n"; }
			logMessage("updateUptime, $node, SNMP ERROR errmsg=$SNMP_Simple::errmsg");
			notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		elsif ( $snmpTable{sysObjectID} =~ /unprintable BER type/ 
				or $snmpTable{sysUpTime} =~ /unprintable BER type/ 
				or $snmpTable{ifNumber} =~ /unprintable BER type/
		) {
			$snmpresult = 0;
			logMessage("updateUptime, $node, SNMP ERROR sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
			if ($debug)  { print returnTime." $node SNMP Problem sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		elsif ( $snmpTable{sysObjectID} eq "" 
				and $snmpTable{sysUpTime} eq "" 
				and $snmpTable{ifNumber} eq "" 
		) {
			$snmpresult = 0;
			logMessage("updateUptime, $node, SNMP elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}");
			if ($debug)  { print returnTime." $node elements empty sysObjectID=$snmpTable{sysObjectID} sysUpTime=$snmpTable{sysUpTime} ifNumber=$snmpTable{ifNumber}\n"; }
			$SNMP_Simple::errmsg = "";
			goto END_updateUptime;
		}
		# check if SNMP matches cached stuff but only if the bugger is up ie did the SNMP return something.
		elsif ( ( $snmpTable{sysObjectID} ne $NMIS::systemTable{sysObjectID}
				or $snmpTable{ifNumber} ne $NMIS::systemTable{ifNumber} 
			) and ( 
				$snmpTable{sysObjectID} ne "" 
				and $snmpTable{ifNumber} ne "" 
		) ) {
			$snmpresult = 100;
			checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
		    if ( $snmpTable{ifNumber} ne $NMIS::systemTable{ifNumber} ) {
				logMessage("updateUptime, $node, Number of interfaces changed previously $NMIS::systemTable{ifNumber} now $snmpTable{ifNumber}");
			}
			if ( $snmpTable{sysObjectID} ne $NMIS::systemTable{sysObjectID} ) {
				logMessage("updateUptime, $node, Device type/model changed $NMIS::systemTable{sysObjectID} now $snmpTable{sysObjectID}");
			}
			&createSystemFile($node, $pingresult);
			&createInterfaceFile($node); 
		}
		# SNMP returned some stuff so store and update etc.
		else { 
			$snmpresult = 100;
			checkEvent(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "SNMP Down", details => "");
		    # Read the uptime from the system info file from the last time it was polled
		    $snmpTable{sysUpTime} =~ s/,/ /g;
		    $snmpTable{sysUpTime} =~ s/  / /g;
			$newUpTime = convertUpTime($snmpTable{sysUpTime});
			$oldUpTime = convertUpTime($NMIS::systemTable{sysUpTime});
		
			if ($debug)  { print returnTime." sysUpTime: Old=$NMIS::systemTable{sysUpTime} New=$snmpTable{sysUpTime}\n"; }
		
			if ( $newUpTime < $oldUpTime and $newUpTime ne "" ) {		
				if ($debug)  { print returnTime." NODE RESET: Old sysUpTime=$oldUpTime: New sysUpTime=$newUpTime\n"; }
				notify(node => $node, role => $NMIS::systemTable{roleType}, type => $NMIS::systemTable{nodeType}, event => "Node Reset", details => "Old_sysUpTime=$NMIS::systemTable{sysUpTime} New_sysUpTime=$snmpTable{sysUpTime}");
			}
		
			$NMIS::systemTable{oldsysUpTime} = $NMIS::systemTable{sysUpTime};
			$NMIS::systemTable{sysUpTime} = $snmpTable{sysUpTime};
			## KS 16/4/2001 - Found that if any details change in nodes.dat not updating system
			$NMIS::systemTable{netType} = $NMIS::nodeTable{$node}{net};
			$NMIS::systemTable{roleType} = $NMIS::nodeTable{$node}{role};
			$NMIS::systemTable{nodeType} = $NMIS::nodeTable{$node}{devicetype};
			
			$writeSystem = "true";
			&writeSystemFile($node);
		}
		# Finished with the SNMP
	}
	END_updateUptime:
	if (defined $session) { $session->close(); }
} # end updateUptime

sub writeSystemFile {
	my $node = shift;
	my $key;
	
	if ($debug) { print returnTime." Writing the System File\n"; }
	
	if ( $writeSystem eq "true" ) {
		# Deal with the nasty \n \r im sysdescr just remove the \n to be put back later.
		$NMIS::systemTable{sysDescr} =~ s/,/ /g;
		$NMIS::systemTable{sysDescr} =~ s/\n/ /g;
		$NMIS::systemTable{sysDescr} =~ s/\r/ /g;
		
		# get damn comma's out:
		$NMIS::systemTable{sysUpTime} =~ s/,/ /g;
		$NMIS::systemTable{sysUpTime} =~ s/\n/ /g;
		$NMIS::systemTable{sysUpTime} =~ s/\r/ /g;
		
		## KS 1/5/01 Made a little more flexible.
		open(OUTFILE, ">$NMIS::config{'<nmis_var>'}/$node.dat") or warn returnTime." writeSystemFile, Couldn't open file $NMIS::config{'<nmis_var>'}/$node.dat for writing. $!\n";
		flock(OUTFILE,2);
		foreach $key (keys %NMIS::systemTable) {
			print OUTFILE "$key=$NMIS::systemTable{$key}\n";
		}
		flock(OUTFILE,8);
		close(OUTFILE); 
	}
	else {
	# only print message if supposed to collect and no SNMP available.
		if ( $NMIS::nodeTable{$node}{collect} eq "true" ) {
			logMessage("writeSystemFile, $node, not writing system file writeSystem=$writeSystem as no SNMP to cache");
		}
	}
}

sub updateRRDDB {
	my %arg = @_;
	my $type = $arg{type};
	my $node = $arg{node};
	my $nodeType = $arg{nodeType}; 
	my $group = $arg{group};
	my $ifDescr = $arg{ifDescr}; 
	my $ifType = $arg{ifType}; 
	my $data = $arg{data}; 

	my $database = getRRDFileName(node => $node, group => $group, nodeType => $nodeType, type => $type, ifDescr => $ifDescr);

	my $time  = 30*int(time/30);
	my @options;
	my $ERROR;
	my $operStatus;
	my $index;

	if ($debug>2) { print returnTime." Starting Update Process type=$type\n"; }

	if ( $type eq "reach" ) {
		@options = (
			"-t", "reachability:availability:responsetime:health",
			"N:$reach{reachability}:$reach{availability}:$reach{responsetime}:$reach{health}"
		);
		if ($debug>4) { print "    Update data with $reach{reachability}:$reach{availability}:$reach{responsetime}:$reach{health}\n"; } 
	} # if reach is true baby
	### AS 8 June 2002 - Adding overall network metrics RRD's
	elsif ( $type eq "metrics" ) {
		@options = (
			"-t", "reachability:availability:responsetime:health:status",
			"N:$data->{reachability}:$data->{availability}:$data->{responsetime}:$data->{health}:$data->{status}"
		);
		if ($debug>4) { print "    Update data with $data->{reachability}:$data->{availability}:$data->{responsetime}:$data->{health}:$data->{status}\n"; } 
	} # if metrics is true baby
	# Is it a Cisco Router Health
	elsif ( $type eq "interface" ) {		
		# Calculate Operational Status
		if (	$ifStats{ifOperStatus} == 1 
				or $ifStats{ifOperStatus} == 5 
				or $ifDescr =~ /ser.*[0-9]:.*/i
				or $ifDescr =~ /virtual/i 
				or $ifDescr =~ /BRI[0-9]:[0-9]/i
		) { $operStatus = 100; }
		else { $operStatus = 0; }

		# While updating start calculating the total availability of the device
		$reach{operStatus} = $reach{operStatus} + $operStatus;
		$reach{operCount} = $reach{operCount} + 1;

		if ( $ifStats{ifInOctets} eq "" ) { $ifStats{ifInOctets} = 0 }
		if ( $ifStats{ifOutOctets} eq "" ) { $ifStats{ifOutOctets} = 0 }

		@options = (
			"-t", "ifInOctets:ifOutOctets:ifOperStatus",
			"N:$ifStats{ifInOctets}:$ifStats{ifOutOctets}:$operStatus"
		);
		if ($debug>4) { print "    Update data with $database with $ifStats{ifInOctets}:$ifStats{ifOutOctets}:$operStatus\n"; }
	}
	elsif ( $type eq "packets" ) {		
		@options = (
			"-t", "ifInUcastPkts:ifInNUcastPkts:ifInDiscards:ifInErrors:ifOutUcastPkts:ifOutNUcastPkts:ifOutDiscards:ifOutErrors",
			"N:$ifStats{ifInUcastPkts}:$ifStats{ifInNUcastPkts}:$ifStats{ifInDiscards}:$ifStats{ifInErrors}:$ifStats{ifOutUcastPkts}:$ifStats{ifOutNUcastPkts}:$ifStats{ifOutDiscards}:$ifStats{ifOutErrors}"
		);
		if ($debug>4) { print "    Update data with $database with $ifStats{ifInUcastPkts}:$ifStats{ifInNUcastPkts}:$ifStats{ifInDiscards}:$ifStats{ifInErrors}:$ifStats{ifOutUcastPkts}:$ifStats{ifOutNUcastPkts}:$ifStats{ifOutDiscards}:$ifStats{ifOutErrors}\n"; }
	}
	elsif ( $type eq "nodehealth" and $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ ) {
		@options = (
			"-t", "avgBusy1:avgBusy5:MemoryUsedPROC:MemoryFreePROC:MemoryUsedIO:MemoryFreeIO:bufferElFree:bufferElHit:bufferFail:ipInReceives:ipForwDatagrams:ipInDelivers:ipOutNoRoutes",
			"N:$snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedPROC}:$snmpTable{MemoryFreePROC}:$snmpTable{MemoryUsedIO}:$snmpTable{MemoryFreeIO}:$snmpTable{bufferElFree}:$snmpTable{bufferElHit}:$snmpTable{bufferFail}:$snmpTable{ipInReceives}:$snmpTable{ipForwDatagrams}:$snmpTable{ipInDelivers}:$snmpTable{ipOutNoRoutes}"
		);
		if ($debug>4) { print "    Updating with $database with $snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedPROC}:$snmpTable{MemoryFreePROC}:\n"; }
	}
	elsif ( $type eq "nodehealth" and $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000Sup3/ ) {
		@options = (
			"-t", "avgBusy1:avgBusy5:MemoryUsedDRAM:MemoryFreeDRAM:MemoryUsedMBUF:MemoryFreeMBUF:MemoryUsedCLUSTER:MemoryFreeCLUSTER:sysTraffic:TopChanges",
			"N:$snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedDRAM}:$snmpTable{MemoryFreeDRAM}:$snmpTable{MemoryUsedMBUF}:$snmpTable{MemoryFreeMBUF}:$snmpTable{MemoryUsedCLUSTER}:$snmpTable{MemoryFreeCLUSTER}:$snmpTable{sysTraffic}:$snmpTable{TopChanges}"
		);
		if ($debug>4) { print "    Update data with $database with $snmpTable{avgBusy1}:$snmpTable{avgBusy5}:$snmpTable{MemoryUsedDRAM}:$snmpTable{MemoryFreeDRAM}\n"; }
	}
	elsif ( $type eq "nodehealth" and $NMIS::systemTable{nodeModel} =~ /Accelar/ ) {
		@options = (
			"-t", "rcSysCpuUtil:rcSysSwitchFabricUtil:rcSysBufferUtil:ipInReceives:ipForwDatagrams:ipInDelivers:ipOutNoRoutes",
			"N:$snmpTable{rcSysCpuUtil}:$snmpTable{rcSysSwitchFabricUtil}:$snmpTable{rcSysBufferUtil}:$snmpTable{ipInReceives}:$snmpTable{ipForwDatagrams}:$snmpTable{ipInDelivers}:$snmpTable{ipOutNoRoutes}" #		
			);
		if ($debug>4) { print "    Updating with $database with $snmpTable{rcSysCpuUtil}:$snmpTable{rcSysSwitchFabricUtil}:$snmpTable{rcSysBufferUtil}:$snmpTable{ipInReceives}:$snmpTable{ipForwDatagrams}:$snmpTable{ipInDelivers}:$snmpTable{ipOutNoRoutes}\n"; }# 
	}

	RRDs::update($database,@options);

	if ($ERROR = RRDs::error) {
		logMessage("updateRRDDB, $node, update error database=$database: $ERROR: options = @options");
	}

	if ($debug>1) { print returnTime." Database=$database Model=$NMIS::systemTable{nodeModel}\n"; }
}

### AS 2 June 2002 - createRRRDB now checks if RRD exists and only creates if doesn't exist.
sub createRRDDB {
	my %arg = @_;
	my $type = $arg{type};
	my $node = $arg{node};
	my $nodeType = $arg{nodeType}; 
	my $group = $arg{group};
	my $ifDescr = $arg{ifDescr}; 
	my $ifType = $arg{ifType}; 
	my $ifIndex = $arg{ifIndex}; 
	my $ifSpeed = $arg{ifSpeed}; 

	my $database = getRRDFileName(node => $node, group => $group, nodeType => $nodeType, type => $type, ifDescr => $ifDescr);
	# Does the database exist already?
	if ( -f $database and -r $database and -w $database ) { 
		# nothing!
		if ($debug>2) { print "  Database $database already exists and is read write for you!\n"; }
		return 1;
	}
	# Check if the RRD Database Exists but is ReadOnly
	# Maybe this should check for valid directory or not.
	elsif ( -f $database and not -w $database ) { 
		print "ERROR: Database $database Exists but is readonly to you!\n";
		return 0;
	}
	# It doesn't so create it
	else { 
		my $time  = 30*int(time/30);
		my @options;
		my ($STEP, $RRD1);
		my $START = $time;
	
		#snmpInPkts, snmpOutPkts, snmpOutTraps, snmpInBadCommunityNames
		#snmpInGenErrs, snmpInSetRequests, snmpInGetRequests, snmpInGetNexts
		#snmpOutGenErrs, snmpOutGetRequests, snmpOutGetRequests, snmpOutGetNexts
		if ( $ifSpeed <= 0 ) { $ifSpeed = 1000000000; }
		if ( $ifType =~ /frame-relay/ ) {
			$ifSpeed = 2000000;
		}

		# Check what type of database to create
		# Is it a Generic Interface Type
		if ( $type eq "interface" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:ifInOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifOutOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifOperStatus:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		elsif ( $type eq "packets" ) {
			# If we can collect extra packet stats Nike!  Make an extra RRD for this interface
			# create a max pps number 10000000 / 14880 = approx 600
			my $packets = $ifSpeed / 600;
			@options = (
				"-b", $START, "-s", 300,
				"DS:ifInOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifOutOctets:COUNTER:900:0:$ifSpeed", 
				"DS:ifInUcastPkts:COUNTER:900:0:$packets", 
				"DS:ifInNUcastPkts:COUNTER:900:0:$packets", 
				"DS:ifInDiscards:COUNTER:900:0:$packets", 
				"DS:ifInErrors:COUNTER:900:0:$packets", 
				"DS:ifOutUcastPkts:COUNTER:900:0:$packets", 
				"DS:ifOutNUcastPkts:COUNTER:900:0:$packets", 
				"DS:ifOutDiscards:COUNTER:900:0:$packets", 
				"DS:ifOutErrors:COUNTER:900:0:$packets", 
				"DS:ifOperStatus:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		# Is it a Reachability Type
		elsif ( $type eq "reach" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:reachability:GAUGE:900:0:100", 
				"DS:availability:GAUGE:900:0:100", 
				"DS:responsetime:GAUGE:900:0:U", 
				"DS:health:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		} # Reachability thingy
		### AS 8 June 2002 - Adding overall network metrics RRD's
		elsif ( $type eq "metrics" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:reachability:GAUGE:900:0:100", 
				"DS:availability:GAUGE:900:0:100", 
				"DS:responsetime:GAUGE:900:0:U", 
				"DS:health:GAUGE:900:0:100", 
				"DS:status:GAUGE:900:0:100", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		} # Reachability thingy
		# Is it a Cisco Router Health
		elsif ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CatalystIOS/ and $type eq "nodehealth" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:avgBusy1:GAUGE:900:U:U", 
				"DS:avgBusy5:GAUGE:900:U:U", 
				"DS:MemoryUsedPROC:GAUGE:900:U:U", 
				"DS:MemoryFreePROC:GAUGE:900:U:U", 
				"DS:MemoryUsedIO:GAUGE:900:U:U", 
				"DS:MemoryFreeIO:GAUGE:900:U:U", 
				"DS:bufferElFree:GAUGE:900:U:U", 
				"DS:bufferElHit:COUNTER:900:U:U", 
				"DS:bufferFail:COUNTER:900:U:U", 
				"DS:ipInReceives:COUNTER:900:U:U", 
				"DS:ipForwDatagrams:COUNTER:900:U:U", 
				"DS:ipInDelivers:COUNTER:900:U:U", 
				"DS:ipOutNoRoutes:COUNTER:900:U:U", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		# Is it a Cisco Catalyst 6000 Switch Health Stats
		elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000Sup3/ and $type eq "nodehealth" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:avgBusy1:GAUGE:900:U:U", 
				"DS:avgBusy5:GAUGE:900:U:U", 
				"DS:MemoryUsedDRAM:GAUGE:900:U:U", 
				"DS:MemoryFreeDRAM:GAUGE:900:U:U", 
				"DS:MemoryUsedMBUF:GAUGE:900:U:U", 
				"DS:MemoryFreeMBUF:GAUGE:900:U:U", 
				"DS:MemoryUsedCLUSTER:GAUGE:900:U:U", 
				"DS:MemoryFreeCLUSTER:GAUGE:900:U:U", 
				"DS:sysTraffic:GAUGE:900:U:U", 
				"DS:TopChanges:COUNTER:900:U:U", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}
		## KS 1 Apr 02 - Integrating Phil Reilly's Nortel changes
		#Our good old Accelar
		elsif ( $NMIS::systemTable{nodeModel} =~ /Accelar/ and $type eq "nodehealth" ) {
			@options = (
				"-b", $START, "-s", 300,
				"DS:rcSysCpuUtil:GAUGE:900:U:100", 
				"DS:rcSysSwitchFabricUtil:GAUGE:900:U:100", 
				"DS:rcSysBufferUtil:GAUGE:900:U:100", 
				"DS:ipInReceives:COUNTER:900:U:U", 
				"DS:ipForwDatagrams:COUNTER:900:U:U", 
				"DS:ipInDelivers:COUNTER:900:U:U", 
				"DS:ipOutNoRoutes:COUNTER:900:U:U", 
				"RRA:AVERAGE:0.5:1:2304",
				"RRA:AVERAGE:0.5:6:1536",
				"RRA:AVERAGE:0.5:24:2268",
				"RRA:AVERAGE:0.5:288:1890",
				"RRA:MAX:0.5:1:2304",
				"RRA:MAX:0.5:6:1536",
				"RRA:MAX:0.5:24:2268",
				"RRA:MAX:0.5:288:1890",
				"RRA:MIN:0.5:1:2304",
				"RRA:MIN:0.5:6:1536",
				"RRA:MIN:0.5:24:2268",
				"RRA:MIN:0.5:288:1890"
			);
		}

		if ($debug) { print returnTime." Creating RRD $database starting at $time.\n"; }
		logMessage("createRRDDB, $node, Creating RRD $database starting at $time");
		RRDs::create("$database",@options);
		my $ERROR = RRDs::error;
		if ($ERROR) {
			logMessage("createRRDDB, $node, unable to create $database: $ERROR");
			return 0;
		}
		return 1;
	} # else
} # end createRRDDB

# Create the Interface configuration from SNMP Stuff!!!!!
sub createInterfaceFile {
	my $node = shift;
	my $interfacefile = "$NMIS::config{'<nmis_var>'}/$node-interface.dat";
	if ($debug) { print returnTime." Creating the interface file: $interfacefile\n"; }
	if ($debug) { print "\tNote the MIB variable 1.3.6.1.2.1.31.1.1.1.1 is the ifName MIB which is not supported by older SNMP agents.\n"; }
	
	# Create a means to maintain which devices are supported so far.
	my $validModel = "false";
	my @ifNames;
	my $ifIndex;
	my $ourPortName;
	my $ourPortNumber;
	my @intTable;
	my $interfaces;
	my $intf;
	my $ifIndex;
	my %interfaceTable;
	my $session;
	my $message;
	my %ifTypeDefs;
	
	my $numInterfaces = $NMIS::systemTable{ifNumber};
	
	#NMIS::loadInterfaceTypes;
	%ifTypeDefs = &loadCSV($NMIS::config{ifTypes_Table},$NMIS::config{ifTypes_Key},"\t");
	
	if ( $NMIS::systemTable{snmpVer} eq "SNMPv2" ) {
		($session) = SNMPv2c_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	else {
		($session) = SNMP_Simple->open($node, $NMIS::nodeTable{$node}{community}, $NMIS::nodeTable{$node}{snmpport});
	}
	if ( not defined($session) ) { 
		warn returnTime." createInterfaceFile, Session is not Defined Goodly to $node.\n"; 
		goto END_createInterfaceFile;
	}
	
	# An SNMP session exists already and we know the node type
	# Depending on the node type get different information
	if ( $NMIS::systemTable{nodeModel} =~ /CiscoRouter|CiscoATM|CatalystIOS/ ) {
		if ($debug) { print "\tCollecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		# Loop to get interface information for Cisco router
		#for ( $index = 1; $index <= $numInterfaces; ++$index ) {
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			# Get Interface Information
			(	$interfaces->{$index}{ifDescr},
				$interfaces->{$index}{ifType},
				$interfaces->{$index}{ifSpeed},
				$interfaces->{$index}{ifAdminStatus},
				$interfaces->{$index}{ifOperStatus},
				$interfaces->{$index}{ifLastChange},
				$interfaces->{$index}{Description}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index",
				'ifAlias'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			}
			if ( $interfaces->{$index}{ifDescr} eq "" ) { 
				$interfaces->{$index}{ifDescr} = "null";
				++$numInterfaces; 
			}
		 
			if ($debug>3) { print "\tifIndex=$index descr=$interfaces->{$index}{ifDescr} admin=$interfaces->{$index}{ifAdminStatus} oper=$interfaces->{$index}{ifOperStatus} speed=$interfaces->{$index}{ifSpeed}\n"; }
			
			# Set the ifType to be something meaningful!!!!
			$interfaces->{$index}{ifType} = $ifTypeDefs{$interfaces->{$index}{ifType}}{ifType};
			
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces->{$index}{ifAdminStatus} = $interfaceStatus[$interfaces->{$index}{ifAdminStatus}];
			$interfaces->{$index}{ifOperStatus} = $interfaceStatus[$interfaces->{$index}{ifOperStatus}];
			
			# Just check if it is an Frame Relay sub-interface
			if ( ( $interfaces->{$index}{ifType} eq "frame-relay" and $interfaces->{$index}{ifDescr} =~ /\./ ) ) {
				$interfaces->{$index}{ifType} = "frame-relay-subinterface";
			}
	} # FOR LOOP
	} # ( $NMIS::systemTable{nodeModel} eq "CiscoRouter" or CiscoATM)
	elsif (	$NMIS::systemTable{nodeModel} =~ /Catalyst6000/ or
			$NMIS::systemTable{nodeModel} =~ /Catalyst5000/
	) {
		$validModel = "true";
		my @ENUMportType;
		$ENUMportType[18] = "e10a100BaseTX";
		$ENUMportType[27] = "e1000BaseLX";
		$ENUMportType[28] = "e1000BaseSX";
		$ENUMportType[31] = "empty";
		$ENUMportType[32] = "e1000BaseLH";
		my @ENUMportSpantreeFastStart = ("null","enabled","disabled");
		my @ENUMportOperStatus = ("null","other","ok","minorFault","majorFault");
		my @ENUMportDuplex = ("null","half","full","disagree","auto");
		
		# Get a list of interface ifIndexes and ifName mappings
		@intTable = $session->snmpgettablek('ifIndex', 'ifName', 'ifAdminStatus', 'ifLastChange', 'ifSpeed');
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
			$SNMP_Simple::errmsg = "";
			logMessage("createInterfaceFile, $message");
			if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
			goto END_createInterfaceFile;
		}
		if ($debug) { print "\tCatalyst Found $#intTable interfaces\n"; }
		
		foreach $ifIndex (1 .. $#intTable) {
			if ($intTable[$ifIndex]->{'ifName'} ne "" ) {
				if ($debug>3) { print "\t$index,$ifIndex,$intTable[$ifIndex]->{'ifIndex'},$intTable[$ifIndex]->{'ifName'}\n"; }

				$intTable[$ifIndex]->{'ifName'} =~ s/,/:/g;
				# Assign table into normalised interfaceTable 
				$interfaces->{$ifIndex}{ifIndex} = $intTable[$ifIndex]->{'ifIndex'};
				$interfaces->{$ifIndex}{ifDescr} = $intTable[$ifIndex]->{'ifName'};
				$interfaces->{$ifIndex}{ifAdminStatus} = $intTable[$ifIndex]->{'ifAdminStatus'};
				$interfaces->{$ifIndex}{ifSpeed} = $intTable[$ifIndex]->{'ifSpeed'};
				$interfaces->{$ifIndex}{ifLastChange} = $intTable[$ifIndex]->{'ifLastChange'};
	
			# if the Interface is a num/num ie a catalyst port then get additional stats
				if ($intTable[$ifIndex]->{ifName} =~ /^\d{1,2}\/\d{1,2}/ ) {
			    if ($debug>3) { print "\tget more details: index=$ifIndex\tifName=$intTable[$ifIndex]->{'ifName'}\tifDescr=$interfaces->{$ifIndex}{ifDescr}\n"; }
					$ourPortNumber =  $intTable[$ifIndex]->{ifName};
					$ourPortNumber =~ s/\//\./;
					
					(       
						$interfaces->{$ifIndex}{Description}, 
						$interfaces->{$ifIndex}{ifType}, 
						$interfaces->{$ifIndex}{ifOperStatus}, 
						$interfaces->{$ifIndex}{portDuplex},
						$interfaces->{$ifIndex}{portSpantreeFastStart},
						$interfaces->{$ifIndex}{vlanPortVlan},
						$interfaces->{$ifIndex}{portAdminSpeed}
					) = $session->snmpget(
						'portName'.".$ourPortNumber", 
						'portType'.".$ourPortNumber", 
						'portOperStatus'.".$ourPortNumber", 
						'portDuplex'.".$ourPortNumber",
						'portSpantreeFastStart'.".$ourPortNumber",
						'vlanPortVlan'.".$ourPortNumber",
						'portAdminSpeed'.".$ourPortNumber"
					);
					if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
						$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
						$SNMP_Simple::errmsg = "";
						logMessage("createInterfaceFile, $message");
						if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
						goto END_createInterfaceFile;
					}
					
					# Turn Enumerated types into words for each value
					# portOperStatus ENUMs
					$interfaces->{$ifIndex}{ifOperStatus} = $ENUMportOperStatus[$interfaces->{$ifIndex}{ifOperStatus}];
					
					# portDuplex ENUMs
					$interfaces->{$ifIndex}{portDuplex} = $ENUMportDuplex[$interfaces->{$ifIndex}{portDuplex}];
					
					# portSpantreeFastStart ENUMs
					$interfaces->{$ifIndex}{portSpantreeFastStart} = $ENUMportSpantreeFastStart[$interfaces->{$ifIndex}{portSpantreeFastStart}]; 
					
					# portType ENUMs
					$interfaces->{$ifIndex}{ifType} = $ENUMportType[$interfaces->{$ifIndex}{ifType}];
					# Because auto is shown as a speed of 1 make it change.
					# portAdminSpeed ENUMs
					if ( $interfaces->{$ifIndex}{portAdminSpeed} == 1 ) {
						$interfaces->{$ifIndex}{portAdminSpeed} = "auto"; 
					}
				} # if interface is num/num
				# must be a vlan or GEC or something else.
				else {
					if ( 	$interfaces->{$ifIndex}{ifDescr} =~ /vlan/i ) {
						$interfaces->{$ifIndex}{ifType} = "VLAN";
						$interfaces->{$ifIndex}{ifSpeed} = "1000000000";
					}
					elsif ( $interfaces->{$ifIndex}{ifDescr} =~ /gec/i ) {
						$interfaces->{$ifIndex}{ifType} = "GIG EtherChannel";
					}
					elsif ( $interfaces->{$ifIndex}{ifDescr} =~ /sl0/i ) {
						$interfaces->{$ifIndex}{ifType} = "SLIP";
					}
					elsif ( $interfaces->{$ifIndex}{ifDescr} =~ /sc0/i ) {
						$interfaces->{$ifIndex}{ifType} = "Management Ethernet";
					}
					$interfaces->{$ifIndex}{ifSpeed} = $intTable[$ifIndex]->{'ifSpeed'};
					$interfaces->{$ifIndex}{ifOperStatus} = $intTable[$ifIndex]->{'ifOperStatus'};
					
					$interfaces->{$ifIndex}{ifOperStatus} = $interfaceStatus[$interfaces->{$ifIndex}{ifOperStatus}];
				} # else other
				$interfaces->{$ifIndex}{ifAdminStatus} = $interfaceStatus[$interfaces->{$ifIndex}{ifAdminStatus}];
		    } # if name is not ""
		} # Foreach loop
	}
	elsif ( $NMIS::systemTable{nodeModel} =~ /SunSolaris|Win2000|generic/ ) {
		if ($debug) { print "Collecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		
		# Loop to get interface information from SNMP
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			if ($debug>1) { print "\tGetting $index, ". 'ifDescr'.".$index" . "\n"; }
			# Get Interface Information
			(       $interfaces->{$index}{ifDescr},
				$interfaces->{$index}{ifType},
				$interfaces->{$index}{ifSpeed},
				$interfaces->{$index}{ifAdminStatus},
				$interfaces->{$index}{ifOperStatus},
				$interfaces->{$index}{ifLastChange}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			}
			if ( $interfaces->{$index}{ifDescr} eq "" ) { 
				$interfaces->{$index}{ifDescr} = "null";
				++$numInterfaces; 
			}
			 
			if ($debug) { print "\tifIndex=$index descr=$interfaces->{$index}{ifDescr} admin=$interfaces->{$index}{ifAdminStatus} oper=$interfaces->{$index}{ifOperStatus} speed=$interfaces->{$index}{ifSpeed}\n"; }
			
			# Set the ifType to be something meaningful!!!!
			$interfaces->{$index}{ifType} = $ifTypeDefs{$interfaces->{$index}{ifType}}{ifType};
	
			# Interfaces with N/A in them are bad!
			if ( $interfaces->{$index}{ifSpeed} eq "0" and $interfaces->{$index}{ifType} =~ /ethernet/i ) {
				$interfaces->{$index}{ifSpeed} = 10000000;
			}
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces->{$index}{ifAdminStatus} = $interfaceStatus[$interfaces->{$index}{ifAdminStatus}];
			$interfaces->{$index}{ifOperStatus} = $interfaceStatus[$interfaces->{$index}{ifOperStatus}];
		} # FOR LOOP
	} # Sun or generic
	## KS 1 Apr 02 - Integrating Phil Reilly's Nortel changes
	elsif ( $NMIS::systemTable{nodeModel} =~ /Accelar/ ) {
		if ($debug) { print "Collecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		
		# Loop to get interface information from SNMP
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			# Get Interface Information
			(       $interfaces->{$index}{ifDescr},
				$interfaces->{$index}{ifType},
				$interfaces->{$index}{ifSpeed},
				$interfaces->{$index}{ifAdminStatus},
				$interfaces->{$index}{ifOperStatus},
				$interfaces->{$index}{ifLastChange},
				$interfaces->{$index}{rcPortType}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index",
				'rcPortType'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			};
			if ( $interfaces->{$index}{ifDescr} eq "" ) { 
				$interfaces->{$index}{ifDescr} = "null";
				++$numInterfaces; 
			}else{
				SWITCH:{
					($index <=31) && do { 
						my $AccelarPort= $index-15;
						$interfaces->{$index}{ifDescr} = "Slot 1 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=32) && ($index <= 47)) && do  {
						my $AccelarPort= $index-31;
						$interfaces->{$index}{ifDescr} = "Slot 2 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=48) && ($index <=63 )) && do {
						my $AccelarPort= $index-47;
						$interfaces->{$index}{ifDescr} = "Slot 3 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=64) && ($index <=79)) && do {
						my $AccelarPort= $index-64;
						$interfaces->{$index}{ifDescr} = "Slot 4 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=80) && ($index <= 95)) && do {
						my $AccelarPort= $index-79;
						$interfaces->{$index}{ifDescr} = "Slot 5 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=96) && ($index <= 111)) && do {
						my $AccelarPort= $index-95;
						$interfaces->{$index}{ifDescr} = "Slot 6 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=112) && ($index <= 127)) && do {
						my $AccelarPort= $index-111;
						$interfaces->{$index}{ifDescr} = "Slot 7 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					(($index >=128) && ($index <= 143)) && do {
						my $AccelarPort= $index-127;
						$interfaces->{$index}{ifDescr} = "Slot 8 Port $AccelarPort";
						++$numInterfaces;
						last SWITCH;};
					$interfaces->{$index}{ifDescr} = "IfIndex $index";
					++$numInterfaces; 
					};
				};
			
			# Set the ifType to be something meaningful!!!! 
			# This is special for Accelars
			SWITCH: {
	  			($interfaces->{$index}{rcPortType} == 6) && do { 
					$interfaces->{$index}{ifType}="10BaseF(mm)";
                        		last SWITCH; };
	  			($interfaces->{$index}{rcPortType} == 5) && do { 
					$interfaces->{$index}{ifType}="1000BaseF(Dual Connector)";
                        		last SWITCH; };
	  			($interfaces->{$index}{rcPortType} == 4) && do { 
					$interfaces->{$index}{ifType}="1000BaseF(mm)";
                        		last SWITCH; };
	  			($interfaces->{$index}{rcPortType} == 3) && do { 
					$interfaces->{$index}{ifType}="100BaseF(mm)";
                        		last SWITCH; };
	  			($interfaces->{$index}{rcPortType} == 2) && do { 
					$interfaces->{$index}{ifType}="100BaseT2(cat3)";
                        		last SWITCH; };
	  			($interfaces->{$index}{rcPortType} == 1) && do { 
					$interfaces->{$index}{ifType}="100BaseTX(cat5)";
                        		last SWITCH; };
				$interfaces->{$index}{ifType}="null"; #else null
				};
			if ($debug) { print "\tifIndex=$index descr=$interfaces->{$index}{ifDescr} admin=$interfaces->{$index}{ifAdminStatus} oper=$interfaces->{$index}{ifOperStatus} speed=$interfaces->{$index}{ifSpeed} rcPortType=$interfaces->{$index}{rcPortType} ($interfaces->{$index}{ifType})\n"; }
			# Interfaces with N/A in them are bad!
			if ( $interfaces->{$index}{ifSpeed} eq "0" and $interfaces->{$index}{ifType} =~ /ethernet/i ) {
				$interfaces->{$index}{ifSpeed} = 10000000;
			};
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces->{$index}{ifAdminStatus} = $interfaceStatus[$interfaces->{$index}{ifAdminStatus}];
			$interfaces->{$index}{ifOperStatus} = $interfaceStatus[$interfaces->{$index}{ifOperStatus}];
		} # FOR LOOP
	} # Accelar or Nortel
	elsif ( $NMIS::systemTable{nodeModel} =~ /BayStack/ ) {
		if ($debug) { print "Collecting $NMIS::systemTable{ifNumber} Interfaces \n"; }
		$validModel = "true";
		
		# Loop to get interface information from SNMP
		my %ifIndexTable = $session->snmpgettablean('ifIndex');
		foreach $index (keys %ifIndexTable) {
			# Get Interface Information
			(       $interfaces->{$index}{ifDescr},
				$interfaces->{$index}{ifType},
				$interfaces->{$index}{ifSpeed},
				$interfaces->{$index}{ifAdminStatus},
				$interfaces->{$index}{ifOperStatus},
				$interfaces->{$index}{ifLastChange}
			) = $session->snmpget(  
				'ifDescr'.".$index",
				'ifType'.".$index",
				'ifSpeed'.".$index",
				'ifAdminStatus'.".$index",
				'ifOperStatus'.".$index",
				'ifLastChange'.".$index"
			); 
			if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
				$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
				$SNMP_Simple::errmsg = "";
				logMessage("createInterfaceFile, $message");
				if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
				goto END_createInterfaceFile;
			}
			if (($interfaces->{$index}{ifDescr} eq "") || ($interfaces->{$index}{ifDescr} =~ /not present/i)){ 
				$interfaces->{$index}{ifDescr} = "null";
				++$numInterfaces; 
			}elsif ($NMIS::systemTable{nodeModel} =~ /303|304|310|Generic/) {
				$interfaces->{$index}{ifDescr} = "Port $index";
				++$numInterfaces;
			}
			 
			if ($debug) { print "\tifIndex=$index descr=$interfaces->{$index}{ifDescr} admin=$interfaces->{$index}{ifAdminStatus} oper=$interfaces->{$index}{ifOperStatus} speed=$interfaces->{$index}{ifSpeed}\n"; }
			
			# Set the ifType to be something meaningful!!!!
			$interfaces->{$index}{ifType} = $ifTypeDefs{$interfaces->{$index}{ifType}}{ifType};
	
			# Interfaces with N/A in them are bad!
			#Also Speed of 200Mb are usually 100FDX.
			if ( $interfaces->{$index}{ifSpeed} eq "200000000"){
				$interfaces->{$index}{ifSpeed} = 100000000;}
			if ( $interfaces->{$index}{ifSpeed} eq "0" and $interfaces->{$index}{ifType} =~ /ethernet/i ) {
				$interfaces->{$index}{ifSpeed} = 10000000;
			}
			# Set AdminStatus and OperStatus to be a word!!
			$interfaces->{$index}{ifAdminStatus} = $interfaceStatus[$interfaces->{$index}{ifAdminStatus}];
			$interfaces->{$index}{ifOperStatus} = $interfaceStatus[$interfaces->{$index}{ifOperStatus}];
		} # FOR LOOP
	} # BayStack
	else {
		$validModel eq "false"; 
	}
	# IP Address Table
	#ip.ipAddrTable.ipAddrEntry.ipAdEntAddr.10.64.100.145 = ipAdEntAddr: 10.64.100.145
	#ip.ipAddrTable.ipAddrEntry.ipAdEntIfIndex.10.64.100.145 = 4
	#ip.ipAddrTable.ipAddrEntry.ipAdEntNetMask.10.64.100.145 = ipAdEntAddr: 255.255.255.248
	#get the IP addressing stuff
	if ( $validModel eq "true") {
		my %ifIndexTable = $session->snmpgettablean('ipAdEntIfIndex');
		my %ifMaskTable = $session->snmpgettablean('ipAdEntNetMask');
		if ( $SNMP_Simple::errmsg =~ /No answer from/ ) {
			$message = "$node SNMP error. errmsg=$SNMP_Simple::errmsg";
			$SNMP_Simple::errmsg = "";
			logMessage("createInterfaceFile, $message");
			if ($debug) { print returnTime." createInterfaceFile, $message\n"; }
			goto END_createInterfaceFile;
		}
		foreach $index (keys %ifIndexTable) {
			if ($debug) { 	
				print "\tip addresses: index=$index ifIndex=$ifIndexTable{$index} mask=$ifMaskTable{$index}\n";	
			}
			$interfaces->{$ifIndexTable{$index}}{ipAdEntAddr} = $index;
			$interfaces->{$ifIndexTable{$index}}{ipAdEntNetMask} = $ifMaskTable{$index};
			($interfaces->{$ifIndexTable{$index}}{ipSubnet},$interfaces->{$ifIndexTable{$index}}{ipSubnetBits}) = ipSubnet(address => $index, mask => $ifMaskTable{$index});
		}
	}
	
	# Process the table and make any changes necessary
	if ( $validModel eq "true") {
		# Now we have a nicely populated Interface Table lets write it out to a file
		if ($debug) { print returnTime." Interface List for $node\n"; }
	
	    foreach $index (keys %$interfaces) {
			if ($debug>1) { print "\tChecking $index, ifDescr=$interfaces->{$index}{ifDescr}\n"; }
			if ( $interfaces->{$index}{ifDescr} ne "" ) {
				#Decide if the interface is one that we can do stats on or not based on Description and ifType and AdminStatus
				# If the interface is admin down no stats
				if (	$interfaces->{$index}{ifAdminStatus} eq "down" or
						$interfaces->{$index}{ifAdminStatus} eq "testing" or
						$interfaces->{$index}{ifAdminStatus} eq "null" or
						$interfaces->{$index}{ifDescr} eq "null" or
						$collect eq "false" 
				) {
					$interfaces->{$index}{collect} = "false";
				} 
				# if it is a router and the interface name has any of these strings
				elsif ( $NMIS::systemTable{nodeModel} =~ /router|ciscoatm/i ) {
					if (
						$interfaces->{$index}{ifDescr} =~ /$qr_no_collect_ifDescr_gen/i
						# Found problems with Cisco ATM interfaces on new IOS
						or $interfaces->{$index}{ifDescr} =~ /$qr_no_collect_ifDescr_atm/i
						# Nothing to collect on Voice interfaces
						or $interfaces->{$index}{ifDescr} =~ /$qr_no_collect_ifDescr_voice/i
						# or if the interface is of this type
						or $interfaces->{$index}{ifType} =~ /$qr_no_collect_ifType_gen/i
					) { $interfaces->{$index}{collect} = "false"; }
					else { $interfaces->{$index}{collect} = "true"; }
				}
				elsif ( $NMIS::systemTable{nodeModel} =~ /CatalystIOS/ ) {
					if (	$interfaces->{$index}{Description} eq "" or
							$interfaces->{$index}{ifType} !~ /ethernet/i
					) { $interfaces->{$index}{collect} = "false"; }
					else { $interfaces->{$index}{collect} = "true"; }
				}
				# if it is a catalyst and the interface name has any of these strings
				elsif ( $NMIS::systemTable{nodeModel} =~ /Catalyst6000|Catalyst5000/ and $interfaces->{$index}{ifDescr} !~ /gec/i ) {
					if (    
						# has to have the port up at the time, this will get rid of PC's and servers that have moved
						$interfaces->{$index}{ifOperStatus} eq "other" 
						# has to have one of these words in its description
						## KS 8/4/01 Uncomment for more specific matching of interfaces
						#( $interfaces->{$index}{Description} !~ /trunk|channel|data|link|backbone/i ) ||
						# blank decription and not VLAN or GEC
						or $interfaces->{$index}{Description} eq ""
					) { $interfaces->{$index}{collect} = "false"; }
					else { $interfaces->{$index}{collect} = "true"; }
				}
				elsif ( $NMIS::systemTable{nodeModel} =~ /SunSolaris|Win2000|generic/i ) {
					if ( 	$interfaces->{$index}{ifType} =~ /loopback/i 
						#or $interfaces->{$index}{Description} eq "" 
					) { $interfaces->{$index}{collect} = "false"; }
					else { $interfaces->{$index}{collect} = "true"; }
				} 
				else { $interfaces->{$index}{collect} = "true"; }
				
				# Remove Nasty comma's from Interface Last Change!
				$interfaces->{$index}{ifLastChange} =~ s/,/ /g;
				
				if ($debug) { print "\t$index\t$interfaces->{$index}{'collect'}\t$interfaces->{$index}{'ifDescr'}\t$interfaces->{$index}{'ifType'}\t\"$interfaces->{$index}{'Description'}\"\n"; }
				if ($debug>3) { print "\t$index,$interfaces->{$index}{ifDescr},$interfaces->{$index}{'collect'},$interfaces->{$index}{'ifType'},$interfaces->{$index}{'ifSpeed'},$interfaces->{$index}{'ifAdminStatus'},$interfaces->{$index}{'ifOperStatus'},$interfaces->{$index}{'ifLastChange'},$interfaces->{$index}{'Description'}\n"; }
	
				$intf = convertIfName($interfaces->{$index}{ifDescr});
				$ifIndex = $index;
				$interfaceTable{$ifIndex}{interface} = $intf;
				$interfaceTable{$ifIndex}{ifIndex} = $ifIndex;
				$interfaceTable{$ifIndex}{ifDescr} = rmBadChars($interfaces->{$index}{ifDescr});
				$interfaceTable{$ifIndex}{collect} = $interfaces->{$index}{collect};
				$interfaceTable{$ifIndex}{ifType} = $interfaces->{$index}{ifType};
				$interfaceTable{$ifIndex}{ifSpeed} = $interfaces->{$index}{ifSpeed};
				$interfaceTable{$ifIndex}{ifAdminStatus} = $interfaces->{$index}{ifAdminStatus};
				$interfaceTable{$ifIndex}{ifOperStatus} = $interfaces->{$index}{ifOperStatus};
				$interfaceTable{$ifIndex}{ifLastChange} = $interfaces->{$index}{ifLastChange};
				$interfaceTable{$ifIndex}{Description} = $interfaces->{$index}{Description};
				$interfaceTable{$ifIndex}{portDuplex} = $interfaces->{$index}{portDuplex};
				$interfaceTable{$ifIndex}{portSpantreeFastStart} = $interfaces->{$index}{portSpantreeFastStart};
				$interfaceTable{$ifIndex}{vlanPortVlan} = $interfaces->{$index}{vlanPortVlan};
				$interfaceTable{$ifIndex}{portAdminSpeed} = $interfaces->{$index}{portAdminSpeed};
				$interfaceTable{$ifIndex}{ipAdEntAddr} = $interfaces->{$index}{ipAdEntAddr};
				$interfaceTable{$ifIndex}{ipAdEntNetMask} = $interfaces->{$index}{ipAdEntNetMask};
				$interfaceTable{$ifIndex}{ipSubnet} = $interfaces->{$index}{ipSubnet};
				$interfaceTable{$ifIndex}{ipSubnetBits} = $interfaces->{$index}{ipSubnetBits};
			} # if ifdescr not ""
		} # FOR LOOP

		# Write the interface table out.
		&writeCSV(%interfaceTable,$interfacefile,"\t");
	    #foreach $intf (keys %interfaceTable ) {
		#	print "$intf: $interfaceTable{$intf}{ifDescr} $interfaceTable{$intf}{ifType} $interfaceTable{$intf}{ifAdminStatus}\n";
		#}
	} # if validModel
	else {
		if ($debug) { print returnTime." createInterfaceFile - Node is not a valid model $node\n"; }
		logMessage("createInterfaceFile, $node, createInterfaceFile - Node is not a valid model $node");
	}
	# Finished with the SNMP
	END_createInterfaceFile:
	if (defined $session) { $session->close(); }
} # end createInterfaceFile

sub checkConfig {
	my $check = shift;
	my $change = 0;
	my $conf_key;
	my $file;
	my $conf_count = 0;
	my $weight = 0;
	my $metric = 0;
	
	if ($debug) { print returnTime." Config Checking - this will validate the NMIS config file=$conf\n"; }
	if ( $check eq "check" ) { $change = 0;  }	
	else { $change = 1; }
	
	foreach $conf_key (sort(keys %NMIS::config)) {
		++$conf_count;
		if ($debug > 3) { print "  config item: $conf_key = $NMIS::config{$conf_key}\n"; }
		# Check files in htdocs stuff
		## KS 23/6/01 Removed map for deprecated map icon check.
		#if ( $conf_key =~ /icon|arrow_|map|styles|help_file/ ) {
		if ( $conf_key =~ /icon|arrow_|styles|help_file|plugin_file/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{'<url_base>'}/$NMIS::config{web_root}/;
			if ( not -r $file ) {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file expanded is $file.  The file doesn't exist, check config items \"web_root=$NMIS::config{web_root}\" and \"<url_base>=$NMIS::config{'<url_base>'}\".\n"; }	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a file and the file exists.\n"; } 
			}
		}
		# Check files in htdocs stuff
		elsif ( $conf_key =~ /web_.*_root/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{web_report_root}/$NMIS::config{web_root}/;
			if ( not -d $file ) {
				if ($type eq "config") { createDir($file); }
				else { "  ". warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a directory expanded is $file.  The directory doesn't exist, check config items \"web_root=$NMIS::config{web_root}\" and \"web_reports_root=$NMIS::config{web_report_root}\".\n"; }
			}	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a directory and the file exists.\n"; } 
			}
		}
		# Check base directories for <> config variables
		elsif ( $conf_key =~ /_root|nmis_|<.*>/ and $conf_key !~ /url|_log$|_host/ ) {
			# check if directory exists
			if ( not -d $NMIS::config{$conf_key} ) {
				if ($type eq "config") { createDir($NMIS::config{$conf_key}); }
				else { "  ". warn returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a directory and the directory doesn't exist\n"; }
			}	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a directory and the directory exists.\n"; } 
			}
		}
		# Check files in cgi-bin stuff
		elsif ( $conf_key =~ /^nmis$|^logs$|^admin$|^ip$|^map$|^view$/ ) {
			# check if file exists
			$file = $NMIS::config{$conf_key};
			$file =~ s/$NMIS::config{'<cgi_url_base>'}/$NMIS::config{'<nmis_cgi>'}/;
			if ( not -r $file ) {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file expanded is $file.  The file doesn't exist, check config items \"cgi_url_base=$NMIS::config{'<cgi_url_base>'}\" and \"<nmis_cgi>=$NMIS::config{'<nmis_cgi>'}\".\n"; }	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - expanded $file - this is a file and the file exists.\n"; } 
			}
		}
		# Check straight files 
		elsif ( $conf_key =~ /Table|_file|_log|_conf|AuthUserFile/ or $conf_key eq "file" ) {
			# check if file exists
			if ( not -r $NMIS::config{$conf_key} 
				and $conf_key !~ /Interface_Table|event_log/
			) {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a file and the file doesn't exist\n";
			}	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a file and the file exists.\n"; } 
			}
		}
		elsif ( $conf_key =~ /_mib$/ ) {
			# check if file exists
			if ( not -r "$NMIS::config{mib_root}/$NMIS::config{$conf_key}" ) {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is a OID file and the file doesn't exist in directory $NMIS::config{mib_root}\n";
			}	
			else { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
				if ($debug > 1) { print "  $conf_key = $NMIS::config{$conf_key} - this is a file and the file exists.\n"; } 
			}
		}
		elsif ( $conf_key =~ /^weight_/ ) {
			if ( $NMIS::config{$conf_key} =~ /^\d+\.\d+$/ ) {
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; }
				$weight = $weight + $NMIS::config{$conf_key};
			} else {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is supposed to be a number.\n";
			}
		}
		elsif ( $conf_key =~ /^metric_/ ) {
			if ( $NMIS::config{$conf_key} =~ /^\d+\.\d+$/ ) {
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; }
				$metric = $metric + $NMIS::config{$conf_key};
			} else {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is supposed to be a number.\n";
			}
		}
		# things which should be a number
		elsif ( $conf_key =~ /^ping_|^conf_count|graph_amount|graph_height|graph_factor|graph_width|escalate/ ) {
			if ( $NMIS::config{$conf_key} =~ /^\d+$/ ) {
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; }
			} else {
				warn "  ". returnTime." checkConfig, config item \"$conf_key=$NMIS::config{$conf_key}\" is supposed to be a number.\n";
			}
		}
		# things which don't require tests
		elsif ( $NMIS::config{$conf_key} =~ /true|false/
			or $conf_key =~ /_Title|_Key|domain_name|url_base|view_tables|trap_server|snpp_server|nmis_host|dash_title|^link_|^mail_|^graph_unit|^no_collect_|^ignore_up_down|int_stats|hc_model|^plugin/
		) { 
				if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - OK\n"; } 
		}
		else {
			if ($type eq "config") { print "    $conf_key=$NMIS::config{$conf_key} - NO TEST AVAILABLE\n"; } 
			if ($debug == 9) { print "      no test for config item $conf_key\n"; }
		}
	}
	
	if ( $conf_count ne $NMIS::config{conf_count} ) {
		print "ERROR: NMIS Config file problem; found $conf_count items should be $NMIS::config{conf_count}. Check config against sample config.\n";
	}
	
	if ( $weight != 1 ) {
		warn "  ". returnTime." checkConfig, weight_.* config items do not add up to 1 which represents 100%\n";
	}
	if ( $metric != 1 ) {
		warn "  ". returnTime." checkConfig, metric_.* config items do not add up to 1 which represents 100%\n"; 
	}

	# Do the database directories exist if not make them?
	my $dir;
	if ($debug) { print returnTime." Config Checking - Checking database directories\n"; }
	if ( -d "$NMIS::config{database_root}" ) {
		if ($type eq "config") { 
			createDir("$NMIS::config{database_root}/health");
			createDir("$NMIS::config{database_root}/metrics");
			createDir("$NMIS::config{database_root}/health/generic");
			createDir("$NMIS::config{database_root}/health/router");
			createDir("$NMIS::config{database_root}/health/switch");
			createDir("$NMIS::config{database_root}/health/server");
			createDir("$NMIS::config{database_root}/interface");
			createDir("$NMIS::config{database_root}/interface/generic");
			createDir("$NMIS::config{database_root}/interface/router");
			createDir("$NMIS::config{database_root}/interface/switch");
			createDir("$NMIS::config{database_root}/interface/server"); 
		}
	}
	#######################################################################
	# make sure that the interface directories exist
	#######################################################################
	foreach my $node ( keys (%NMIS::nodeTable) ) {
		if ( $NMIS::nodeTable{$node}{active} ne "false" 
			and $NMIS::nodeTable{$node}{collect} eq "true" 
		) {
			# if the directory for interface don't exist create it 
			if (    not -d "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$node}{devicetype}/$node" 
				and not -r "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$node}{devicetype}/$node" 
				and -w "$NMIS::config{database_root}/interface/$NMIS::nodeTable{$node}{devicetype}" 
			) { 
				logMessage("runNodeStats, $node, creating interface database directory");
				createDir("$NMIS::config{database_root}/interface/$NMIS::nodeTable{$node}{devicetype}/$node");
			}
		}
	}
}

sub runLinks {
	my %links;
	my %subnets;
	loadInterfaceInfo;
	if ($debug) { print returnTime." Auto Generating Links file\n"; }
	foreach my $intHash (sort(keys(%NMIS::interfaceInfo))) {
       	if ( 	$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "" and
       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} ne "0.0.0.0" and
       			$NMIS::interfaceInfo{$intHash}{ipAdEntAddr} !~ /^127/  
		) {
			my $subnet = $NMIS::interfaceInfo{$intHash}{ipSubnet};
			if ( ! defined $subnets{$subnet}{subnet} 
				and $NMIS::interfaceInfo{$intHash}{collect} eq "true" 
				#and $NMIS::interfaceInfo{$intHash}{ifType} =~ /$qr_link_ifType/
			) {
				loadSystemFile($NMIS::interfaceInfo{$intHash}{node});
				$subnets{$subnet}{subnet} = $subnet;
				$subnets{$subnet}{count} = 1;
				$subnets{$subnet}{description} = $NMIS::interfaceInfo{$intHash}{Description};
				$subnets{$subnet}{mask} = $NMIS::interfaceInfo{$intHash}{ipAdEntNetMask};
				$subnets{$subnet}{ifSpeed} = $NMIS::interfaceInfo{$intHash}{ifSpeed};
				$subnets{$subnet}{ifType} = $NMIS::interfaceInfo{$intHash}{ifType};
				$subnets{$subnet}{address1} = $NMIS::interfaceInfo{$intHash}{ipAdEntAddr};
				$subnets{$subnet}{node1} = $NMIS::interfaceInfo{$intHash}{node};
				$subnets{$subnet}{interface1} = $NMIS::interfaceInfo{$intHash}{ifDescr};
				$subnets{$subnet}{net1} = $NMIS::nodeTable{$NMIS::interfaceInfo{$intHash}{node}}{net};
				$subnets{$subnet}{role1} = $NMIS::nodeTable{$NMIS::interfaceInfo{$intHash}{node}}{role};
				$subnets{$subnet}{location1} = $NMIS::systemTable{sysLocation};
				
			}
			elsif ( defined $subnets{$subnet}{subnet} 
				and $NMIS::interfaceInfo{$intHash}{collect} eq "true" 
				#and $NMIS::interfaceInfo{$intHash}{ifType} =~ /$qr_link_ifType/
			) {
				loadSystemFile($NMIS::interfaceInfo{$intHash}{node});
				++$subnets{$subnet}{count};
				if ( ! defined $subnets{$subnet}{description} ) {
					$subnets{$subnet}{description} = $NMIS::interfaceInfo{$intHash}{Description};
				}
				$subnets{$subnet}{address2} = $NMIS::interfaceInfo{$intHash}{ipAdEntAddr};
				$subnets{$subnet}{node2} = $NMIS::interfaceInfo{$intHash}{node};
				$subnets{$subnet}{interface2} = $NMIS::interfaceInfo{$intHash}{ifDescr};				
				$subnets{$subnet}{net2} = $NMIS::nodeTable{$NMIS::interfaceInfo{$intHash}{node}}{net};
				$subnets{$subnet}{role2} = $NMIS::nodeTable{$NMIS::interfaceInfo{$intHash}{node}}{role};
				$subnets{$subnet}{location2} = $NMIS::systemTable{sysLocation};
			} else {
				++$subnets{$subnet}{count};				
			}
		}
	} # foreach
	my $i = 1;
	foreach my $subnet (sort(keys(%subnets))) {
		if ( $subnets{$subnet}{count} == 2 ) {
			if ($debug) { print "   Adding $subnet to links.\n"; }
			if ( defined $subnets{$subnet}{description} 
				and $subnets{$subnet}{description} ne "null"
			) {
				$links{$i}{link} = $subnets{$subnet}{description};
			} else {
				$links{$i}{link} = $i;	
			}
			$links{$i}{subnet} = $subnets{$subnet}{subnet};
			$links{$i}{mask} = $subnets{$subnet}{mask};
			$links{$i}{ifSpeed} = $subnets{$subnet}{ifSpeed};
			$links{$i}{ifType} = $subnets{$subnet}{ifType};
			$links{$i}{net} = $subnets{$subnet}{net1};
			$links{$i}{role} = $subnets{$subnet}{role1};
			$links{$i}{location} = $subnets{$subnet}{location1};
			$links{$i}{node1} = $subnets{$subnet}{node1};
			$links{$i}{interface1} = $subnets{$subnet}{interface1};
			$links{$i}{node2} = $subnets{$subnet}{node2};
			$links{$i}{interface2} = $subnets{$subnet}{interface2};
			$links{$i}{depend} = "N/A";
			++$i;
		} else {
			if ($debug) { print "   $subnet has $subnets{$subnet}{count} nodes.\n"; }
		}
	}
	&writeCSV(%links,$NMIS::config{Links_Table},"\t");
	print returnTime." DONE.  Check file $NMIS::config{Links_Table} and update link names and other entries.\n";
}

### AS 8 June 2002 - Adding overall network metrics collection and updates
sub runMetrics {
	my %groupSummary;
	my $data;
	my $group;
	my $status;

	if ($debug) { print "\n".returnTime." Running metrics!\n"; }

	loadEventState;
	loadNodeDetails;

	# Doing the whole network
	my %groupSummary = &getGroupSummary();
	$status = overallNodeStatus;
	$status = statusNumber($status);
	$data->{reachability} = $groupSummary{average}{reachable};
	$data->{availability} = $groupSummary{average}{available};
	$data->{responsetime} = $groupSummary{average}{response};
	$data->{health} = $groupSummary{average}{health};
	$data->{status} = $status;
	if ($debug>2) { print returnTime." Doing Metrics database! r=$data->{reachability} a=$data->{availability} rt=$data->{responsetime} h=$data->{health} s=$data->{status}\n"; }
	if ( &createRRDDB(type => "metrics", group => "network") ) { 
		&updateRRDDB(type => "metrics",group => "network", data => $data);
	}

	foreach $group (sort ( keys (%NMIS::groupTable) ) ) {
		%groupSummary = getGroupSummary($group);
		$status = overallNodeStatus($group);
		$status = statusNumber($status);
		$data->{reachability} = $groupSummary{average}{reachable};
		$data->{availability} = $groupSummary{average}{available};
		$data->{responsetime} = $groupSummary{average}{response};
		$data->{health} = $groupSummary{average}{health};
		$data->{status} = $status;
		
		if ($debug>2) { print returnTime." Doing Metrics database! r=$data->{reachability} a=$data->{availability} rt=$data->{responsetime} h=$data->{health} s=$data->{status}\n"; }
		if ( &createRRDDB(type => "metrics", group => $group) ) { 
			&updateRRDDB(type => "metrics",group => $group, data => $data);
		}
	}
	
} # end runMetrics

sub createDir {
	my $dir = shift;
	if ( not -d $dir ) { 
		print "  Creating directory $dir\n"; 
		mkdir($dir,0775) or warn "ERROR: cannot mkdir $dir: $!\n"; 
	}
}

sub printApache {
	my $check = shift;
	my $change = 0;
	my $conf_key;
	my $file;
	my @cgi = (
		"nmis",
		"logs",
		"admin",
		"view"
	);
	
	if ($debug) { print returnTime." Apache HTTPD Config for NMIS for config file=$conf\n"; }
	if ( $check eq "check" ) { $change = 0;  }	
	else { $change = 1; }

	print <<EO_TEXT;
## For more information on the listed Apache features read:
## Alias directive:        http://httpd.apache.org/docs/mod/mod_alias.html#alias
## ScriptAlias directive:  http://httpd.apache.org/docs/mod/mod_alias.html#scriptalias
## Order directive:        http://httpd.apache.org/docs/mod/mod_access.html#order
## Allow directive:        http://httpd.apache.org/docs/mod/mod_access.html#allow
## Deny directive:         http://httpd.apache.org/docs/mod/mod_access.html#deny
## AuthType directive:     http://httpd.apache.org/docs/mod/core.html#authtype
## AuthName directive:     http://httpd.apache.org/docs/mod/core.html#authname
## AuthUserFile directive: http://httpd.apache.org/docs/mod/mod_auth.html#authuserfile
## Require directive:      http://httpd.apache.org/docs/mod/core.html#require

# Usual Apache Config File!
#<apache_root>/conf/httpd.conf

# add a password to the users.dat file!
#<apache_root>/bin/htpasswd /usr/local/nmis/conf/users.dat nmis

# restart the daemon!
#<apache_root>/bin/apachectl restart 
#
# NOTE:
# <apache_root> is normally /usr/local/apache
# the "bin" directory might be "sbin"
# the "conf" directory might be "etc"
# the httpd.conf might be split across httpd.conf, access.conf and srm.conf

# NMIS Aliases

Alias $NMIS::config{'<url_base>'}/ "$NMIS::config{web_root}/"
ScriptAlias $NMIS::config{'<cgi_url_base>'}/ "$NMIS::config{'<nmis_cgi>'}/"
ScriptAlias /nmis "$NMIS::config{'<nmis_cgi>'}/nmiscgi.pl"

EO_TEXT

	foreach $conf_key (@cgi) {
		print <<EO_TEXT;
<Location "$NMIS::config{$conf_key}">
	## For IP address based permissions
	## this will allow either any one on:
	## Network 10.0.0.0, network 172.16.0.0, host 192.168.1.1
	## or anyone with the domain .sins.com.au
	## UNCOMMENT THE FOLLOWING 3 LINES TO ENABLE IP ADDRESS AUTHENTICATION
	## ENSURE YOU UNDERSTAND PRIOR TO ENABLING.
	#Order deny,allow
	#deny from all
	#allow from 10.0.0.0/8 172.16.0.0/16 192.168.1.1 .sins.com.au
	#
	## For Username based authentication
	AuthType Basic
	AuthName "NMIS"
	AuthUserFile $NMIS::config{AuthUserFile}
	Require valid-user
</Location>

EO_TEXT
	}
}	

sub weightResponseTime {
	my $rt = shift;
	my $responseWeight = 0;

	if ( $rt eq "" ) { 
		$rt = "U";
		$responseWeight = 0; 
	}
	elsif ( $rt !~ /^[0-9]/ ) { 
		$rt = "U";
		$responseWeight = 0; 
	}
	elsif ( $rt == 0 ) { 
		$rt = 1; 
		$responseWeight = 100; 
	}
	elsif ( $rt >= 1500 ) { $responseWeight = 0; }
	elsif ( $rt >= 1000 ) { $responseWeight = 10; }
	elsif ( $rt >= 900 ) { $responseWeight = 20; }
	elsif ( $rt >= 800 ) { $responseWeight = 30; }
	elsif ( $rt >= 700 ) { $responseWeight = 40; }
	elsif ( $rt >= 600 ) { $responseWeight = 50; }
	elsif ( $rt >= 500 ) { $responseWeight = 60; }
	elsif ( $rt >= 400 ) { $responseWeight = 70; }
	elsif ( $rt >= 300 ) { $responseWeight = 80; }
	elsif ( $rt >= 200 ) { $responseWeight = 90; }
	elsif ( $rt >= 0 ) { $responseWeight = 100; }
	return ($rt,$responseWeight);
}

sub checkArgs {
	if ( $#ARGV < 0 ) {
	print <<EO_TEXT;
$0 NMIS Polling Engine - Network Mangement Information System
Copyright (C) 2000 Sinclair InterNetworking Services Pty Ltd

command line options are:
  type=<option>          Where <option> is one of the following:
                           collect   NMIS will collect all statistics;
                           update    Update all the dynamic NMIS configuration
                           threshold Check collected stats for thresholds
                           config    Validate the chosen configuration file
                           apache    Produce Apache configuration for NMIS
                           links     Generate the links.csv file.
  [file=<file name>]     Optional alternate configuation files;
  [node=<node name>]     Run operations on a single node;
  [debug=true|false|0-9] default=false - Show debuging information, handy;
  [collect=true|false]   default=true - Do the SNMP collection or not;

EO_TEXT
		exit(0);
	}
}