#
#    rrdfunc.pm - func NMIS Perl Package - Network Management Information System
#    Copyright (C) 2000,2001 Sinclair InterNetworking Services Pty Ltd
#    <nmis@sins.com.au> http://www.sins.com.au
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
package rrdfunc;

require 5;

use strict;

use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

use FindBin;
use lib "$FindBin::Bin/../lib";
use lib "/usr/local/rrdtool/lib/perl"; 

use RRDs;
use Statistics::Lite qw(min max range sum count mean median mode variance stddev statshash statsinfo);
use NMIS;
use func;

$VERSION = 1.00;

@ISA = qw(Exporter);

@EXPORT = qw(
		getRRDasHash
		getRRDStats
	);

@EXPORT_OK = qw(	
	);

sub getRRDasHash {
	my %arg = @_;
	my ($begin,$step,$name,$data) = RRDs::fetch($arg{rrd},$arg{type},"--start",$arg{start},"--end",$arg{end});
	my %s;
	my @h;
	my $f = 1;
	my $date;
	my $d;
	my $time = $begin;
	for(my $a = 0; $a <= $#{$data}; ++$a) {
		$d = 0;
		for(my $b = 0; $b <= $#{$data->[$a]}; ++$b) {
			if ($f) { push(@h,$name->[$b]) }
			$s{$time}{$name->[$b]} = $data->[$a][$b];
			if ( defined $data->[$a][$b] ) { $d = 1; }
		}
		if ($d) {
			$date = returnDateStamp($time);
			$s{$time}{time} = $time;
			$s{$time}{date} = $date;
		}
		if ($f) { 
			push(@h,"time");
			push(@h,"date");
		}
		$f = 0;
		$time = $time + $step;
	}
	return (\%s,\@h);
}

sub getRRDStats {
	my %arg = @_;
	if ( ! defined $arg{type} ) { $arg{type} = "AVERAGE"; }
	if ( -r $arg{rrd} ) {
		my ($begin,$step,$name,$data) = RRDs::fetch($arg{rrd},$arg{type},"--start",$arg{start},"--end",$arg{end});
		my %s;
		my $time = $begin;
		for(my $a = 0; $a <= $#{$data}; ++$a) {
			my $date = returnDateStamp($time);
			my $hour = $date;
			my $f = 1;
			$hour =~ s/\d+-[a-zA-Z]+-\d+ (\d+):\d+:\d+/$1/;		
			for(my $b = 0; $b <= $#{$data->[$a]}; ++$b) {
				if ( (	defined $data->[$a][$b] 
						and ! defined $arg{hour_from}
						and ! defined $arg{hour_to}
					) or ( 
						defined $data->[$a][$b] 
						and defined $arg{hour_from} 
						and defined $arg{hour_to} 
						and $hour > $arg{hour_from} 
						and $hour < $arg{hour_to} 
					)
				) {
					if ($f) { 
						#print "$date hour=$hour\n";
						$f = 0;
					}
					push(@{$s{$name->[$b]}{values}},$data->[$a][$b]);
				}
			}
			$time = $time + $step;
		}

		foreach my $m (sort keys %s) {
			$s{$m}{stddev} = sprintf("%.3f",stddev(@{$s{$m}{values}}));
			$s{$m}{mean} = sprintf("%.3f",mean(@{$s{$m}{values}}));
			$s{$m}{median} = sprintf("%.3f",median(@{$s{$m}{values}}));
			$s{$m}{min} = sprintf("%.3f",min(@{$s{$m}{values}}));
			$s{$m}{max} = sprintf("%.3f",max(@{$s{$m}{values}}));
			$s{$m}{range} = sprintf("%.3f",range(@{$s{$m}{values}}));
			$s{$m}{sum} = sprintf("%.3f",sum(@{$s{$m}{values}}));
			$s{$m}{count} = sprintf("%.3f",count(@{$s{$m}{values}}));
			$s{$m}{variance} = sprintf("%.3f",variance(@{$s{$m}{values}}));
		}
		return \%s;
	} else {
		logMessage("getRRDStats,,RRD is not readable rrd=$arg{rrd}");
		return undef;
	}
}

1;
